package com.starbuds.server.api.channel.common.pojo.constant;

import com.starbuds.server.api.channel.common.pojo.constant.notify.NotifyApiDetail;
import com.starbuds.server.api.channel.common.pojo.constant.notify.NotifyParameter;

import java.util.Arrays;
import java.util.List;

/**
 * 异步通知接口
 *
 * @author aoe
 * @date 2020/8/19
 */
public enum NotifyApiEnum {

    SEND_RED_ENVELOPE(sendRedEnvelope("发红包异步通知", 10)),
    ;

    /**
     * 接口名称
     */
    private String apiName;

    /**
     * 接口编号
     */
    private int apiNo;

    /**
     * 接口路径
     */
    private String apiPath;

    /**
     * 异步通知接口描述
     */
    private NotifyApiDetail detail;

    private static final String POST = "POST";
    private static final String CONTENT_TYPE_JSON = "application/json";
    private static final String TYPE_STRING = "String";
    private static final String TYPE_DATE = "Date";

    NotifyApiEnum(NotifyApiDetail detail) {
        this.apiName = detail.getApiName();
        this.apiNo = detail.getApiNo();
        this.apiPath = detail.getApiPath();
        this.detail = detail;
    }

    public String apiName() {
        return apiName;
    }

    public int apiNo() {
        return apiNo;
    }

    public String apiPath() {
        return apiPath;
    }

    public NotifyApiDetail detail() {
        return detail;
    }

    public static NotifyApiEnum toEnum(int apiNo) {
        for (NotifyApiEnum value : values()) {
            if (value.apiNo - apiNo == 0) {
                return value;
            }
        }
        return null;
    }

    /**
     * 发红包
     *
     * @param apiName
     * @param apiNo
     * @return
     */
    private static NotifyApiDetail sendRedEnvelope(String apiName, int apiNo) {
        NotifyApiDetail detail = createDetail(apiName, apiNo);
        detail.setApiDesc("通知调用方发红包处理结果");
        detail.setMethod(POST);
        detail.setContentType(CONTENT_TYPE_JSON);

        // 异步通知公共参数
        List<NotifyParameter> commonParams = Arrays.asList(
                new NotifyParameter("result", "处理结果", TYPE_STRING, true, 7, "success:成功; fail:失败", "success"),
                new NotifyParameter("notify_time", "通知时间", TYPE_DATE, true, 19, "通知的发送时间。格式为yyyy-MM-dd HH:mm:ss", "2015-14-27 05:55:58"),
                new NotifyParameter("sign", "签名", TYPE_STRING, true, 256, "请参考下文的“异步返回结果的验签”", "601510b7970e52cc63db0f44997cf70e")
        );
        detail.setCommonParams(commonParams);

        // 异步通知参数
        List<NotifyParameter> notifyParams = Arrays.asList(
                new NotifyParameter("tradeId", "第三方交易id", TYPE_STRING, true, 100, "发红包时的交易id", "t12345")
        );
        detail.setNotifyParams(notifyParams);

        // 异步返回参数
        List<NotifyParameter> returnParams = Arrays.asList(
                new NotifyParameter("result", "处理结果", TYPE_STRING, true, 7, "success:成功; fail:失败", "success")
        );
        detail.setReturnParams(returnParams);

        // 通知触发条件
        List<String> triggerConditions = Arrays.asList("保存发红包请求成功");
        detail.setTriggerConditions(triggerConditions);

        // CURL请求示例
        detail.setCurlExample("curl --header \"Content-Type: application/json\" \\\n" +
                "  --request POST \\\n" +
                "  --data '{\"result\":\"success\",\"notify_time\":\"2015-14-27 05:55:58\",\"sign\":\"601510b7970e52cc63db0f44997cf70e\"}' \\\n" +
                "  http://localhost:8080/getUserByPhone -t utf-8");

        // 正常响应示例
        detail.setResponseExample("{\"result\": \"success\"}");

        // 异步返回结果的验签说明
        List<String> verificationInstructions = Arrays.asList(
                "签名方法RSA2(key size 2048)",
                "第一步： 在通知返回参数列表中，除去sign这个参数外，凡是通知返回回来的参数皆是待验签的参数。",
                "第二步： 将剩下参数进行字典排序，组成字符串，得到待签名字符串：",
                "body=大乐透2.1&buyer_id=2088102116773037&charset=utf-8&gmt_close=2016-07-19 14:10:46",
                "第三步： 将签名参数（sign）使用base64解码为字节码串。",
                "第四步： 使用RSA的验签方法，通过签名字符串、签名参数（经过base64解码）及星芽公钥验证签名。"
        );
        detail.setVerificationInstructions(verificationInstructions);

        // 失败码说明
        List<String> failCodes = Arrays.asList(
                "10:余额不足",
                "500:服务器内部错误"
        );
        detail.setFailCodes(failCodes);
        return detail;
    }

    /**
     * 创建异步通知接口描述
     *
     * @return
     */
    private static NotifyApiDetail createDetail(String apiName, int apiNo) {
        NotifyApiDetail detail = new NotifyApiDetail();
        detail.setApiName(apiName);
        detail.setApiNo(apiNo);
        detail.setApiPath("xy" + apiNo);
        return detail;
    }
}
