package com.starbuds.server.api.channel.common.service.admin;

import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminSecurityDO;
import com.starbuds.server.api.manage.common.pojo.admin.TokenParam;

/**
 * 安全服务
 *
 * @author Lay
 * @date 2019/12/26
 */
public interface ChAdminSecurityService {
    /**
     * 根据账号查询
     *
     * @param account
     * @return
     */
    ChannelAdminSecurityDO getSecurityByAccount(String account);

    /**
     * 根据账号id查询
     *
     * @param adminId
     * @return
     */
    ChannelAdminSecurityDO getSecurityByAdminId(Long adminId);

    /**
     * 密码检查
     *
     * @param password
     * @param dbPass
     * @return
     */
    boolean checkPass(String password, String dbPass);

    /**
     * 更新登录令牌
     *
     * @param adminId
     * @param clientType
     * @param token
     * @return
     */
    boolean setToken(Long adminId, Integer clientType, String token);

    /**
     * 加密令牌
     *
     * @param tokenParam
     * @return
     */
    String encryptToken(TokenParam tokenParam);

    /**
     * 解密令牌
     *
     * @param tokenEncrypted
     * @return
     */
    TokenParam decryptToken(String tokenEncrypted);

    /**
     * 移除令牌
     *
     * @param adminId
     * @return
     */
    boolean removeToken(Long adminId);

    /**
     * 修改密码
     *
     * @param adminId 管理员id
     * @param newPwd  新密码
     * @return 是否成功
     */
    boolean changePwd(Long adminId, String newPwd);

    /**
     * 是否有渠道权限
     *
     * @param adminId
     * @param channelId
     * @return true 拥有权限
     */
    boolean hasPermissions(Long adminId, Long channelId);
}
