package com.starbuds.server.api.channel.common.service.admin;

import com.starbuds.server.api.channel.common.pojo.admin.ChannelAdminInfoVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelRatioReq;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminDO;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminSecurityDO;
import com.starbuds.server.common.pojo.api.PageList;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 管理员服务
 *
 * @author Lay
 * @date 2019/12/26
 */
public interface ChAdminService {
    /**
     * 获取管理员信息
     *
     * @param adminId
     * @return
     */
    ChannelAdminInfoVO getInfo(Long adminId);

    /**
     * 判断管理员是否存在
     *
     * @param adminId 管理员id
     * @return 是否存在
     */
    boolean isAdminExist(Long adminId);


    /**
     * 创建管理员
     *
     * @param infoVO 管理员
     * @return 是否成功
     */
    boolean createAdmin(ChannelAdminInfoVO infoVO);


    /**
     * 编辑管理员
     *
     * @param adminDO 管理员
     * @return 是否成功
     */
    boolean editAdmin(ChannelAdminDO adminDO);


    /**
     * 禁用管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    boolean disableAdmin(Long adminId);

    /**
     * 启用管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    boolean enableAdmin(Long adminId);

    /**
     * 删除管理员
     *
     * @param adminId 管理员id
     * @return 是否成功
     */
    boolean deleteAdmin(Long adminId);

    /**
     * 获取管理员列表
     *
     * @param adminName  管理员姓名
     * @param adminPhone 手机号
     * @param status     状态
     * @param pageIndex  当前页码
     * @param pageSize   分页大小
     * @param adminId    管理员id
     * @return result对象
     */
    PageList<ChannelAdminInfoVO> getAdminPageList(String adminName, String adminPhone, Integer status, int pageIndex, int pageSize, Long adminId);

    /**
     * 判断是否存在账号
     *
     * @param adminPhone 手机号
     * @return 是否存在
     */
    boolean isPhoneExist(String adminPhone);

    /**
     * 获取admin状态
     *
     * @param adminPhone
     * @return
     */
    Integer getAdminStatus(String adminPhone);

    /**
     * 获取管理员id
     *
     * @param adminPhone
     * @return
     */
    Long getAdminId(String adminPhone);

    /**
     * 是否可用
     *
     * @param adminId
     * @return
     */
    boolean isAdminEnable(Long adminId);

    /**
     * 获取渠道id
     *
     * @param adminId
     * @return
     */
    Long getChannelId(Long adminId);

    /**
     * 获取渠道分组id
     *
     * @param adminId
     * @return
     */
    Long getChannelGroupId(Long adminId);

    /**
     * 更新
     *
     * @param securityDO
     * @return
     */
    boolean updateSecurity(ChannelAdminSecurityDO securityDO);

    /**
     * 是否是上级id
     *
     * @param adminId
     * @return
     */
    boolean isParent(Long adminId);

    /**
     * 设置渠道分成比例
     *
     * @param ratioReq
     * @return
     */
    boolean setChannelRatio(ChannelRatioReq ratioReq);

    /**
     * 获取授权渠道id
     *
     * @param adminId
     * @return
     */
    String getAuthChannelIds(Long adminId);

    /**
     * 获取子管理员
     *
     * @param adminId
     * @return
     */
    List<Long> getAdminIdByParentId(Long adminId);

    /**
     * 获取上级id
     *
     * @param adminId
     * @return
     */
    Long getParentId(Long adminId);

    /**
     * 获取管理员
     *
     * @param adminId
     * @return
     */
    ChannelAdminDO getAdmin(Long adminId);

    /**
     * 获取管理员Map
     *
     * @param adminIds 管理员id集合
     * @return
     */
    Map<Long, ChannelAdminDO> getAdminMapByIds(Set<Long> adminIds);
}
