package com.starbuds.server.api.channel.common.service.admin;

import com.starbuds.server.api.channel.common.pojo.admin.ChannelAppVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelDTO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelInfoVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelSecretVO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.ChannelDO;

import java.util.List;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/4/28 14:41
 * @Description 渠道业务
 */
public interface ChChannelService {

    /**
     * 获取渠道信息
     *
     * @param channelId
     * @return
     */
    ChannelInfoVO getInfo(Long channelId);

    /**
     * 渠道是否存在
     *
     * @param channelId
     * @return
     */
    boolean existByChannelId(Long channelId);

    /**
     * 更新渠道信息
     *
     * @param channelDO
     * @return
     */
    int update(ChannelDO channelDO);

    /**
     * 获得渠道 APP 信息
     *
     * @param channelId
     * @return
     */
    ChannelAppVO getChannelApp(Long channelId);

    /**
     * 查询渠道下秘钥的数量
     *
     * @param channelId
     * @return
     */
    int getSecretKeyCountByChannelId(Long channelId);

    /**
     * 根据ChannelId查询已启用的秘钥数量
     *
     * @param channelId
     * @return
     */
    int getActivatedCount(Long channelId);

    /**
     * 创建渠道秘钥
     *
     * @param channelId
     * @return
     */
    String createSecretKey(long channelId);

    /**
     * 改变渠道秘钥
     *
     * @param channelId
     * @param secretId
     * @return
     */
    String changeSecretKey(Long channelId, Long secretId);

    /**
     * 删除渠道APP秘钥
     *
     * @param channelId
     * @param secretId
     * @return
     */
    int deleteSecretKey(Long channelId, Long secretId);

    /**
     * 启用/禁用渠道APP秘钥
     *
     * @param channelId
     * @param secretId
     * @param isActivate 1:启用; 0:禁用
     * @return
     */
    int activateSecretKey(Long channelId, Long secretId, int isActivate);

    /**
     * 查询所有未删除秘钥
     *
     * @param channelId
     * @return
     */
    List<ChannelSecretVO> getSecretKeyList(Long channelId);

    /**
     * 获取分组渠道集合
     *
     * @return
     */
    List<ChannelDTO> getGroupChannelList(Long channelGroupId);

    /**
     * 获取渠道列表
     *
     * @param channelIdSet
     * @param channelName
     * @param status
     * @param pageIndex
     * @param pageSize
     * @return
     */
    PageList<ChannelInfoVO> getChannelPageList(Set<Long> channelIdSet, String channelName, Integer status, int pageIndex, int pageSize);

    /**
     * 获取渠道
     *
     * @param authChannelIdSet
     * @return
     */
    List<ChannelDTO> getChannelList(Set<Long> authChannelIdSet);

    /**
     * 获得正在使用的秘钥
     *
     * @param channelId 渠道Id
     * @return
     */
    String getActivateSecretKey(Long channelId);
}
