package com.starbuds.server.api.channel.common.service.admin;

import com.starbuds.server.api.channel.common.pojo.admin.*;
import com.starbuds.server.common.pojo.api.PageList;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/2/18 15:29
 * @Description 平台统计业务接口
 */
public interface ChStatPlatformService {

    /**
     * 获取平台统计列表
     *
     * @param channelIds
     * @param statType
     * @param statTimeType
     * @param pageIndex
     * @param pageSize
     * @return
     */
    PageList<StatPlatformVO> getStatPlatformPageList(Set<Long> channelIds,
                                                     Integer statType,
                                                     Integer statTimeType,
                                                     int pageIndex,
                                                     int pageSize);


    /**
     * 获取平台统计
     *
     * @param channelId
     * @return
     */
    List<CurrentAndYesterdayStatEntity> getCurrentAndYesterdayStat(Long channelId);

    /**
     * 获取统计详情
     *
     * @param channelId
     * @param timeType
     * @param startTime
     * @param endTime
     * @param statType
     * @return
     */
    StatDetail getStatDetail(Long channelId, Integer timeType, Long startTime, Long endTime, Integer statType);

    /**
     * 时段概况  顶部
     *
     * @param channelIdArray
     * @param startDate
     * @param endDate
     * @return
     */
    List<StatPeriodOverViewTop> getPeriodOverviewTop(String channelIdArray, Long startDate, Long endDate);

    /**
     * 时段概况  中间
     *
     * @param channelIdArray
     * @param startDate
     * @param endDate
     * @param tagType
     * @return
     */
    StatPeriodOverViewMiddle getPeriodOverviewMiddle(String channelIdArray, Long startDate, Long endDate, Integer tagType);

    /**
     * 时段概况  底部
     *
     * @param channelIdArray
     * @param startDate
     * @param endDate
     * @return
     */
    Map<String, List<StatPeriodOverViewBottom>> getPeriodOverviewBottom(String channelIdArray, Long startDate, Long endDate);

    /**
     * 整体趋势 顶部
     *
     * @param channelIdSet
     * @param startDate
     * @param endDate
     * @return
     */
    List<OverallTrendTop> getOverallTrendTop(Set<Long> channelIdSet, Long startDate, Long endDate);

    /**
     * 整体趋势 中间
     *
     * @param channelIdArray
     * @param startDate
     * @param endDate
     * @param tagType
     * @return
     */
    OverallTrendMiddle getOverallTrendMiddle(String channelIdArray, Long startDate, Long endDate, Integer tagType);

    /**
     * 整体趋势 底部
     *
     * @param channelIdSet
     * @param startDate
     * @param endDate
     * @return
     */
    Map<Long, List<OverallTrendBottom>> getOverallTrendBottom(Set<Long> channelIdSet, Long startDate, Long endDate);

    /**
     * 用户访问趋势
     *
     * @param channelIdArray
     * @param startDate
     * @param endDate
     * @return
     */
    Map<Long, List<UserAccessTrend>> getUserAccessTrend(String channelIdArray, Long startDate, Long endDate);

    /**
     * 用户付费趋势
     *
     * @param channelIdArray
     * @param startDate
     * @param endDate
     * @return
     */
    Map<Long, List<UserPaymentTrend>> getUserPaymentTrend(String channelIdArray, Long startDate, Long endDate);

    /**
     * 渠道注册趋势
     *
     * @param channelIdArray
     * @param startDate
     * @param endDate
     * @return
     */
    Map<Long, List<ChannelRegisterTrend>> getChannelRegisterTrend(String channelIdArray, Long startDate, Long endDate);


}
