package com.starbuds.server.api.channel.service.admin.dao;

import com.starbuds.server.api.channel.common.pojo.admin.ChannelAdminInfoVO;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminDO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-01-15 14:12:01
 * @Description 家族管理员持久层
 */
@Mapper
@Repository
public interface ChannelAdminDAO {
    /**
     * 新增
     *
     * @param entity
     * @return
     */
    int insert(ChannelAdminDO entity);

    /**
     * 根据主键更新
     *
     * @param entity
     * @return
     */
    int updateById(ChannelAdminDO entity);

    /**
     * 根据id获取vo
     *
     * @param adminId
     * @return
     */
    ChannelAdminInfoVO selectVOById(@Param("adminId") Long adminId);


    /**
     * 查询adminId是否存在
     *
     * @param adminId
     * @return
     */
    int countById(@Param("adminId") Long adminId);

    /**
     * 更新状态
     *
     * @param adminId
     * @param adminStatus
     * @return
     */
    int updateStatus(@Param("adminId") Long adminId, @Param("adminStatus") Integer adminStatus);

    /**
     * 获取管理员列表
     *
     * @param adminName   管理员姓名
     * @param adminPhone  售价好
     * @param adminStatus 状态
     * @param pageCursor  光标
     * @param pageSize    分页大小
     * @param adminId     管理员id
     * @return 管理员列表
     */
    List<ChannelAdminInfoVO> getAdminPageList(@Param("adminName") String adminName,
                                              @Param("adminPhone") String adminPhone,
                                              @Param("adminStatus") Integer adminStatus,
                                              @Param("pageCursor") int pageCursor,
                                              @Param("pageSize") int pageSize,
                                              @Param("adminId") Long adminId);

    /**
     * 获取管理员列表大小
     *
     * @param adminName   管理员姓名
     * @param adminPhone  售价好
     * @param adminStatus 状态
     * @param adminId     管理员id
     * @return 列表大小
     */
    int countAdminPageList(@Param("adminName") String adminName,
                           @Param("adminPhone") String adminPhone,
                           @Param("adminStatus") Integer adminStatus,
                           @Param("adminId") Long adminId);

    /**
     * 获取管理员状态
     *
     * @param adminId
     * @return
     */
    Integer getAdminStatus(@Param("adminId") Long adminId);

    /**
     * 获取渠道id
     *
     * @param adminId
     * @return
     */
    Long selectChannelId(@Param("adminId") Long adminId);

    /**
     * 获取渠道分组id
     *
     * @param adminId
     * @return
     */
    Long selectChannelGroupId(@Param("adminId") Long adminId);

    /**
     * 获取状态
     *
     * @param adminPhone
     * @return
     */
    Integer selectAdminStatus(@Param("adminPhone") String adminPhone);

    /**
     * 获取管理员id
     *
     * @param adminPhone
     * @return
     */
    Long selectAdminId(@Param("adminPhone") String adminPhone);

    /**
     * 判断是否上级id
     *
     * @param adminId
     * @return
     */
    int isParent(@Param("adminId") Long adminId);

    /**
     * 获取管理员名
     *
     * @param adminId
     * @return
     */
    String selectAdminName(@Param("adminId") Long adminId);

    /**
     * 获取渠道授权id
     *
     * @param adminId
     * @return
     */
    String selectAuthChannelIds(@Param("adminId") Long adminId);

    /**
     * 获取母账号下的子账号
     *
     * @param parentId
     * @return
     */
    List<Long> selectAdminIdByParentId(@Param("parentId") Long parentId);

    /**
     * 获取上级id
     *
     * @param adminId
     * @return
     */
    Long selectParentId(@Param("adminId") Long adminId);

    /**
     * 获取管理员
     *
     * @param adminId
     * @return
     */
    ChannelAdminDO selectAdmin(@Param("adminId") Long adminId);

    /**
     * 获取管理员Map
     *
     * @param adminIds 管理员id集合
     * @return
     */
    @MapKey("adminId")
    Map<Long, ChannelAdminDO> selectAdminMapByIds(@Param("adminIds") Set<Long> adminIds);
}
