package com.starbuds.server.api.channel.service.admin.dao;

import com.starbuds.server.api.channel.common.pojo.admin.ChannelDTO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelInfoVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelRatioReq;
import com.starbuds.server.common.pojo.db.ChannelDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/4/28 14:47
 * @Description 渠道持久层
 */
@Mapper
@Repository
public interface ChannelDAO {

    /**
     * 获取渠道信息
     *
     * @param channelId
     * @return
     */
    ChannelInfoVO getInfo(@Param("channelId") Long channelId);

    /**
     * 渠道是否存在
     *
     * @param channelId
     * @return
     */
    int existByChannelId(@Param("channelId") Long channelId);

    /**
     * 获得渠道 APP 信息
     *
     * @param channelId
     * @return
     */
    ChannelDO selectChannelApp(@Param("channelId") Long channelId);

    /**
     * 更新数据
     */
    int update(ChannelDO channel);

    /**
     * 获取渠道id
     *
     * @param channelGroupId
     * @return
     */
    List<Long> selectByGroupId(@Param("channelGroupId") Long channelGroupId);

    /**
     * 获取渠道DTO
     *
     * @param channelGroupId
     * @return
     */
    List<ChannelDTO> selectListByChannelGroupId(@Param("channelGroupId") Long channelGroupId);

    /**
     * 获取渠道
     *
     * @param channelIdSet
     * @return
     */
    List<ChannelDTO> selectChannels(@Param("channelIdSet") Set<Long> channelIdSet);

    /**
     * 更新分成比例
     *
     * @param ratioReq
     * @return
     */
    int updateRatio(ChannelRatioReq ratioReq);

    /**
     * 获取渠道列表
     *
     * @param channelIdSet
     * @param channelName
     * @param status
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<ChannelInfoVO> selectChannelPageList(@Param("channelIdSet") Set<Long> channelIdSet,
                                              @Param("channelName") String channelName,
                                              @Param("status") Integer status,
                                              @Param("pageCursor") int pageCursor,
                                              @Param("pageSize") int pageSize);

    /**
     * 获取渠道
     *
     * @param channelIdSet
     * @param channelName
     * @param status
     * @return
     */
    int countChannelPageList(@Param("channelIdSet") Set<Long> channelIdSet,
                             @Param("channelName") String channelName,
                             @Param("status") Integer status);

}
