package com.starbuds.server.api.channel.service.admin.dao;


import com.starbuds.server.api.channel.common.pojo.admin.ChannelSecretVO;
import com.starbuds.server.common.pojo.db.ChannelSecretDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 渠道(APP)秘钥
 */
@Mapper
@Repository
public interface ChannelSecretDAO {

    /**
     * 添加数据
     */
    int insert(ChannelSecretDO channelAuth);

    /**
     * 更新数据
     */
    int update(ChannelSecretDO channelAuth);

    /**
     * 根据主键、ChannelId更新数据
     */
    int updateByPkChannelId(ChannelSecretDO channelAuth);

    /**
     * 更新除secretId以外的其他数据为禁用状态
     *
     * @param channelAuth
     * @return
     */
    int updateDisable(ChannelSecretDO channelAuth);

    /**
     * 更新秘钥
     *
     * @param secret
     * @return
     */
    int updateSecretKey(ChannelSecretDO secret);

    /**
     * 根据ChannelId查询秘钥数量
     *
     * @param channelId
     * @return
     */
    int selectCountByChannelId(@Param("channelId") Long channelId);

    /**
     * 根据ChannelId查询已启用的秘钥数量
     *
     * @param channelId
     * @return
     */
    int selectActivatedCount(@Param("channelId") Long channelId);

    /**
     * 查询所有未删除秘钥
     *
     * @param channelId
     * @return
     */
    List<ChannelSecretVO> selectList(@Param("channelId") Long channelId);

    /**
     * 根据主键查询数据
     */
    ChannelSecretDO selectByPrimaryKey(Long secretId);

    /**
     * 查询列表总数
     */
    int selectCountPageList();

    /**
     * 查询列表(分页)
     */
    List<ChannelSecretDO> selectPageList(@Param("pageCursor") int pageCursor,
                                         @Param("pageSize") int pageSize);

    /**
     * 获得正在使用的秘钥
     *
     * @param channelId 渠道id
     * @return
     */
    String selectActivateSecretKey(@Param("channelId") Long channelId);
}
