package com.starbuds.server.api.channel.service.admin.dao;

import com.starbuds.server.api.channel.common.pojo.admin.CurrentAndYesterdayStatEntity;
import com.starbuds.server.api.channel.common.pojo.admin.StatChannelDetail;
import com.starbuds.server.api.channel.common.pojo.admin.StatPlatformVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-02-18 15:28:11
 * @Description 平台数据统计持久层
 */
@Mapper
@Repository
public interface StatPlatformDAO {


    /**
     * 获取平台统计列表
     *
     * @param channelIds
     * @param statType
     * @param statTimeType
     * @param pageCursor
     * @param pageSize
     * @return
     */
    List<StatPlatformVO> getStatPlatformPageList(@Param("channelIds") Set<Long> channelIds,
                                                 @Param("statType") Integer statType,
                                                 @Param("statTimeType") Integer statTimeType,
                                                 @Param("pageCursor") int pageCursor,
                                                 @Param("pageSize") int pageSize);

    /**
     * 获取平台统计列表大小
     *
     * @param channelIds
     * @param statType
     * @param statTimeType
     * @return
     */
    Integer countStatPlatformPageList(@Param("channelIds") Set<Long> channelIds,
                                      @Param("statType") Integer statType,
                                      @Param("statTimeType") Integer statTimeType);


    /**
     * 查询当天和昨天的数据
     *
     * @param statTypes
     * @return
     */
    List<CurrentAndYesterdayStatEntity> selectCurrentAndYesterdayValue(@Param("statTypes") String statTypes);

    /**
     * 根据类型获取统计详情
     *
     * @return
     */
    List<StatChannelDetail> selectStatDetail(@Param("channelId") Long channelId,
                                             @Param("statType") Integer statType,
                                             @Param("startTime") Long startTime,
                                             @Param("endTime") Long endTime);

    /**
     * 根据时间查询
     *
     * @param dateList
     * @param channelId
     * @return
     */
    List<StatPlatformVO> selectByStatTime(@Param("dateList") List<Long> dateList, @Param("channelId") Long channelId);

    /**
     * 获取统计
     *
     * @param channelIds
     * @param statTypes
     * @param statTimeType
     * @param startDate
     * @param endDate
     */
    List<StatPlatformVO> selectStat(@Param("channelIds") List<Long> channelIds,
                                    @Param("statTypes") List<Integer> statTypes,
                                    @Param("statTimeType") Integer statTimeType,
                                    @Param("startDate") Long startDate,
                                    @Param("endDate") Long endDate);

    /**
     * 时段概况 中间
     *
     * @param channelIds
     * @param statTypes
     * @param statTimeType
     * @param datetimes
     * @return
     */
    List<StatPlatformVO> selectStatForPeriodOverviewMiddle(@Param("channelIds") List<Long> channelIds,
                                                           @Param("statTypes") List<Integer> statTypes,
                                                           @Param("statTimeType") Integer statTimeType,
                                                           @Param("datetimes") Set<Long> datetimes);
}
