package com.starbuds.server.api.channel.service.admin.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.channel.common.pojo.constant.NotifyApiEnum;
import com.starbuds.server.api.channel.common.pojo.constant.notify.NotifyApiDetail;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelNotifyConfigDetailVO;
import com.starbuds.server.common.pojo.db.ChannelNotifyConfigDO;
import com.starbuds.server.api.channel.service.admin.dao.ChannelNotifyConfigDAO;
import com.starbuds.server.api.channel.common.pojo.qo.ChNotifyConfigPageListQO;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelNotifyConfigVO;

import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;

import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.List;

import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyConfigService;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Service;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 渠道异步通知配置服务
 */
@Service
@Slf4j
public class ChChannelNotifyConfigServiceImpl implements ChChannelNotifyConfigService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "chNotifyCfgId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ChannelNotifyConfigDAO channelNotifyConfigDAO;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ChannelNotifyConfigDO create(ChannelNotifyConfigDO po) {
        po.setChNotifyCfgId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = channelNotifyConfigDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ChannelNotifyConfigDO> list) {
        boolean flag = channelNotifyConfigDAO.insertBatch(list) > 0;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ChannelNotifyConfigDO po) {
        boolean flag = channelNotifyConfigDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ChannelNotifyConfigDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getChNotifyCfgId());
        uw.setQw(qw);
        return update(uw);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = channelNotifyConfigDAO.update(uw) > 0;
        return flag;
    }

    @Override
    public ChannelNotifyConfigDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelNotifyConfigDAO.selectOne(qw);
    }

    @Override
    public ChannelNotifyConfigVO getVOById(Long id) {
        ChannelNotifyConfigDO po = getById(id);
        if (po == null) {
            return null;
        }
        ChannelNotifyConfigVO vo = new ChannelNotifyConfigVO();
        BeanUtils.copyProperties(po, vo);
        return vo;
    }

    @Override
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelNotifyConfigDAO.selectCount(qw) > 0;
    }

    @Override
    public PageList<ChannelNotifyConfigDO> getPageList(ChNotifyConfigPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ChannelNotifyConfigDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(channelNotifyConfigDAO.selectPageList(qw));
        page.setTotal(channelNotifyConfigDAO.selectCount(qw));
        return page;
    }

    @Override
    public PageList<ChannelNotifyConfigVO> getVOPageList(ChNotifyConfigPageListQO qo) {
        PageList<ChannelNotifyConfigVO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        PageList<ChannelNotifyConfigDO> poPage = getPageList(qo);
        page.setTotal(poPage.getTotal());

        List<ChannelNotifyConfigVO> voList = new ArrayList<>(poPage.getList().size());
        for (ChannelNotifyConfigDO po : poPage.getList()) {
            ChannelNotifyConfigVO vo = new ChannelNotifyConfigVO();
            BeanUtils.copyProperties(po, vo);
            voList.add(vo);
        }
        page.setList(voList);
        return page;
    }

// 手动编码开始

    /**
     * 获得渠道异步通知配置详情
     */
    @Override
    public ChannelNotifyConfigDetailVO getNotifyCfgVO(long channelId, NotifyApiEnum notifyApiEnum) {
        ChannelNotifyConfigDetailVO vo = new ChannelNotifyConfigDetailVO();
        ChannelNotifyConfigDO po = getNotifyCfg(channelId, notifyApiEnum);
        BeanUtils.copyProperties(po, vo);
        vo.setDetail(notifyApiEnum.detail());
        return vo;
    }

    /**
     * 获得渠道异步通知配置
     *
     * @param channelId
     * @param notifyApiEnum
     * @return
     */
    @Override
    public ChannelNotifyConfigDO getNotifyCfg(long channelId, NotifyApiEnum notifyApiEnum) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("channelId", channelId);
        qw.eq("apiNo", notifyApiEnum.apiNo());
        ChannelNotifyConfigDO po = channelNotifyConfigDAO.selectOne(qw);

        if (po == null) {
            po = new ChannelNotifyConfigDO();
            po.setChannelId(channelId);
            po.setApiNo(notifyApiEnum.apiNo());
            po = create(po);
        }
        return po;
    }

    /**
     * 获得Api地址
     *
     * @param channelId
     * @param notifyApiEnum
     * @return
     */
    @Override
    public String getApiUrl(long channelId, NotifyApiEnum notifyApiEnum) {
        ChannelNotifyConfigDO po = getNotifyCfg(channelId, notifyApiEnum);
        return po.getHostUrl() + notifyApiEnum.apiPath();
    }
}