package com.starbuds.server.api.channel.service.admin.impl;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.channel.common.pojo.constant.NotifyApiEnum;
import com.starbuds.server.api.channel.common.pojo.db.ChannelSecretRsaDO;
import com.starbuds.server.api.channel.common.pojo.qo.ChNotifyHisPageListQO;
import com.starbuds.server.api.channel.common.pojo.request.NotifyRequest;
import com.starbuds.server.api.channel.common.pojo.request.RedEnvelopeNotifyRequest;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelNotifyHisVO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyConfigService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyHisService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyResendTempService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelSecretRsaService;
import com.starbuds.server.api.channel.service.admin.dao.ChannelNotifyHisDAO;
import com.starbuds.server.api.channel.util.ChannelOpenApiSingUtil;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.ChannelNotifyHisDO;
import com.starbuds.server.common.pojo.db.ChannelNotifyResendTempDO;
import com.starbuds.server.common.pojo.db.ChannelRedEnvelopeHisDO;
import com.starbuds.server.common.service.core.IdentityService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 渠道异步通知记录服务
 */
@Service
@Slf4j
public class ChChannelNotifyHisServiceImpl implements ChChannelNotifyHisService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ChannelNotifyHisDAO channelNotifyHisDAO;

    @Resource
    private ChChannelNotifyConfigService chChannelNotifyConfigService;

    @Resource
    private ChChannelNotifyResendTempService chChannelNotifyResendTempService;

    @Resource
    private ChChannelSecretRsaService chChannelSecretRsaService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ChannelNotifyHisDO create(ChannelNotifyHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = channelNotifyHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ChannelNotifyHisDO> list) {
        boolean flag = channelNotifyHisDAO.insertBatch(list) > 0;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ChannelNotifyHisDO po) {
        boolean flag = channelNotifyHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ChannelNotifyHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = channelNotifyHisDAO.update(uw) > 0;
        return flag;
    }

    @Override
    public ChannelNotifyHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelNotifyHisDAO.selectOne(qw);
    }

    @Override
    public ChannelNotifyHisVO getVOById(Long id) {
        ChannelNotifyHisDO po = getById(id);
        if (po == null) {
            return null;
        }
        ChannelNotifyHisVO vo = new ChannelNotifyHisVO();
        BeanUtils.copyProperties(po, vo);
        return vo;
    }

    @Override
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelNotifyHisDAO.selectCount(qw) > 0;
    }

    @Override
    public PageList<ChannelNotifyHisDO> getPageList(ChNotifyHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ChannelNotifyHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(channelNotifyHisDAO.selectPageList(qw));
        page.setTotal(channelNotifyHisDAO.selectCount(qw));
        return page;
    }

    @Override
    public PageList<ChannelNotifyHisVO> getVOPageList(ChNotifyHisPageListQO qo) {
        PageList<ChannelNotifyHisVO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        PageList<ChannelNotifyHisDO> poPage = getPageList(qo);
        page.setTotal(poPage.getTotal());

        List<ChannelNotifyHisVO> voList = new ArrayList<>(poPage.getList().size());
        for (ChannelNotifyHisDO po : poPage.getList()) {
            ChannelNotifyHisVO vo = new ChannelNotifyHisVO();
            BeanUtils.copyProperties(po, vo);
            voList.add(vo);
        }
        page.setList(voList);
        return page;
    }

// 手动编码开始

    /**
     * 发送红包回调通知
     *
     * @param channelId
     * @param traceId
     * @param apiEnum
     * @param request   渠道异步通知请求
     * @param times     请求次数
     */
    @Override
    public void sendNotification(long channelId, Long traceId, NotifyApiEnum apiEnum, NotifyRequest request, int times) {
        request.setNotify_time(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(System.currentTimeMillis()));
        ChannelSecretRsaDO rsa = chChannelSecretRsaService.getRsa(channelId);
        String sign = null;
        try {
            sign = ChannelOpenApiSingUtil.rsaSign(JSON.parseObject(JSON.toJSONString(request)), rsa.getXyPrivKey());
        } catch (Exception e) {
            log.error("发送红包回调通知失败，签名失败 traceId=" + traceId, e);
        }
        request.setSign(sign);
        String notifyData = JSON.toJSONString(request);

        String url = chChannelNotifyConfigService.getApiUrl(channelId, apiEnum);
        if (url == null) {
            url = "没有设置回调地址";
        }

        ChannelNotifyHisDO po = new ChannelNotifyHisDO();
        po.setTraceId(traceId);
        po.setChannelId(channelId);
        po.setApiNo(apiEnum.apiNo());
        po.setNotifyUrl(url);
        po.setNotifyData(notifyData);
        po.setNotifyTimes(times);
        po.setIsSuccess("success".equals(request.getResult()) ? BoolIntVal.True : BoolIntVal.False);

        long startTime = System.currentTimeMillis();
        String res = null;
        try {
            res = HttpUtil.post(url, notifyData, 5000);
        } catch (Exception e) {
            log.error("发送回调通知失败", e);
        }
        po.setCallDuration((int) (System.currentTimeMillis() - startTime));
        po.setResponse(res);
        int resIsSuccess = BoolIntVal.False;
        if (StringUtils.isNotBlank(res) && JSON.isValidObject(res)) {
            JSONObject json = JSON.parseObject(res);
            if ("success".equals(json.getString("result"))) {
                resIsSuccess = BoolIntVal.True;
            }
        }
        po.setResIsSuccess(resIsSuccess);

        create(po);

        // 失败重发，最多重试5次
        if (po.getResIsSuccess() != BoolIntVal.True && po.getNotifyTimes() < 5) {
            ChannelNotifyResendTempDO resend = new ChannelNotifyResendTempDO();
            BeanUtils.copyProperties(po, resend);
            resend.setNotifyTimes(po.getNotifyTimes());
            resend.setTradeId(request.getTradeId());
            chChannelNotifyResendTempService.create(resend);
        }
    }

    /**
     * 重发通知消息
     *
     * @param hisId 渠道异步通知记录id
     */
    @Override
    public void resendNotification(Long hisId, ChannelNotifyHisDO notifyHisDO, ChannelRedEnvelopeHisDO redEnvHis) {
        ChannelNotifyHisDO po = getById(hisId);
        RedEnvelopeNotifyRequest request = new RedEnvelopeNotifyRequest(redEnvHis.getTradeId(), po.getIsSuccess() == BoolIntVal.True);
        sendNotification(po.getChannelId(), redEnvHis.getHisId(), NotifyApiEnum.SEND_RED_ENVELOPE, request, 1);
    }

}