package com.starbuds.server.api.channel.service.admin.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.channel.common.pojo.qo.ChNotifyResendTempPageListQO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyResendTempService;
import com.starbuds.server.api.channel.service.admin.dao.ChannelNotifyResendTempDAO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.ChannelNotifyResendTempDO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

import java.util.List;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 渠道回调通知（失败后）重发临时记录服务
 */
@Service
@Slf4j
public class ChChannelNotifyResendTempServiceImpl implements ChChannelNotifyResendTempService {

    @Resource
    private ChannelNotifyResendTempDAO channelNotifyResendTempDAO;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ChannelNotifyResendTempDO create(ChannelNotifyResendTempDO po) {
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = channelNotifyResendTempDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

// 手动编码开始

    @Override
    public List<ChannelNotifyResendTempDO> getResendList(ChNotifyResendTempPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        return channelNotifyResendTempDAO.selectPageList(qw);
    }

    /**
     * 删除已重新发送的列表
     *
     * @param qo
     * @return
     */
    @Override
    public int deleteResendList(ChNotifyResendTempPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        return channelNotifyResendTempDAO.deleteResendList(qw);
    }
}