package com.starbuds.server.api.channel.service.admin.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.channel.common.pojo.qo.ChRechargeHisPageListQO;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelRechargeHisVO;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelRechargeVO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelRechargeHisService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelWalletService;
import com.starbuds.server.api.channel.service.admin.dao.ChannelRechargeHisDAO;
import com.starbuds.server.common.constant.common.CurrencyEnum;
import com.starbuds.server.common.constant.finance.PayStatusEnum;
import com.starbuds.server.common.constant.finance.PayWayEnum;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.ChannelRechargeHisDO;
import com.starbuds.server.common.pojo.finance.ChannelPayOrder;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.PayService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 渠道充值记录服务
 */
@Service
@Slf4j
public class ChChannelRechargeHisServiceImpl implements ChChannelRechargeHisService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ChannelRechargeHisDAO channelRechargeHisDAO;

    @Reference(version = "1.0.0")
    private PayService payService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ChannelRechargeHisDO create(ChannelRechargeHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = channelRechargeHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ChannelRechargeHisDO> list) {
        boolean flag = channelRechargeHisDAO.insertBatch(list) > 0;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ChannelRechargeHisDO po) {
        boolean flag = channelRechargeHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ChannelRechargeHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = channelRechargeHisDAO.update(uw) > 0;
        return flag;
    }

    @Override
    public ChannelRechargeHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelRechargeHisDAO.selectOne(qw);
    }

    @Override
    public ChannelRechargeHisVO getVOById(Long id) {
        ChannelRechargeHisDO po = getById(id);
        if (po == null) {
            return null;
        }
        ChannelRechargeHisVO vo = new ChannelRechargeHisVO();
        BeanUtils.copyProperties(po, vo);
        return vo;
    }

    @Override
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelRechargeHisDAO.selectCount(qw) > 0;
    }

    @Override
    public PageList<ChannelRechargeHisDO> getPageList(ChRechargeHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        qw.apply("his.hisId = pay.orderId");
        PageList<ChannelRechargeHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(channelRechargeHisDAO.selectPageList(qw));
        page.setTotal(channelRechargeHisDAO.selectCount(qw));
        return page;
    }

    @Override
    public PageList<ChannelRechargeHisVO> getVOPageList(ChRechargeHisPageListQO qo) {
        PageList<ChannelRechargeHisVO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        PageList<ChannelRechargeHisDO> poPage = getPageList(qo);
        page.setTotal(poPage.getTotal());

        List<ChannelRechargeHisVO> voList = new ArrayList<>(poPage.getList().size());
        for (ChannelRechargeHisDO po : poPage.getList()) {
            ChannelRechargeHisVO vo = new ChannelRechargeHisVO();
            BeanUtils.copyProperties(po, vo);
            voList.add(vo);
        }
        page.setList(voList);
        return page;
    }

// 手动编码开始

    /**
     * 渠道钱包充值
     *
     * @param channelId
     * @param amount
     * @return
     */
    @Override
    public ChannelRechargeVO recharge(Long channelId, Long amount) {
        ChannelRechargeHisDO po = new ChannelRechargeHisDO();
        po.setChannelId(channelId);
        po.setPayAmount(new BigDecimal(amount.toString()));
        po.setPayCurrency(CurrencyEnum.RMB.value());
        // 星钻对RMB比例 100:1
        po.setCoinAmount(po.getPayAmount().multiply(new BigDecimal("100")).longValue());
        po.setCoinType(WalletTypeEnum.STAR_DIAMOND.getValue());

        po = create(po);
        if (po == null) {
            return null;
        }
        ChannelRechargeVO vo = new ChannelRechargeVO();
        ChannelPayOrder payOrder = payService.addChannelPayOrder(po.getHisId(), channelId, po.getPayAmount(),
                PayWayEnum.ALI_QR.getValue());
        BeanUtils.copyProperties(payOrder, vo);
        vo.setOrderId(po.getHisId().toString());

        return vo;
    }

    /**
     * 查询渠道充值订单支付状态
     *
     * @param channelId
     * @param hisId
     * @return
     */
    @Override
    public Integer getPayStatus(Long channelId, Long hisId) {
        ChannelRechargeHisDO po = getById(hisId);
        if (po == null) {
            log.warn("无效的充值记录 hisId={}", hisId);
            return null;
        }
        if (channelId - po.getChannelId() != 0) {
            log.warn("查询订单支付状态渠道id不一致 用户渠道Id {} 流水渠道Id {}", channelId, po.getChannelId());
            return null;
        }
        return channelRechargeHisDAO.selectPayStatus(hisId);
    }

}