package com.starbuds.server.api.channel.service.admin.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.starbuds.server.api.channel.common.pojo.bo.RedEnvelopeDetail;
import com.starbuds.server.api.channel.common.pojo.constant.NotifyApiEnum;
import com.starbuds.server.api.channel.common.pojo.constant.SendRedEnvStatusEnum;
import com.starbuds.server.api.channel.common.pojo.db.ChannelAdminDO;
import com.starbuds.server.api.channel.common.pojo.qo.ChRedEnvelopeHisPageListQO;
import com.starbuds.server.api.channel.common.pojo.request.RedEnvelopeNotifyRequest;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelRedEnvelopeHisVO;
import com.starbuds.server.api.channel.common.pojo.vo.SendRedEnvelopeDetailVO;
import com.starbuds.server.api.channel.common.service.admin.ChAdminService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyHisService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelRedEnvelopeHisService;
import com.starbuds.server.api.channel.service.admin.dao.ChannelRedEnvelopeHisDAO;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.ChannelRedEnvelopeHisDO;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.finance.ChannelExchangeOrder;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.ChannelWalletService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 渠道发红包记录服务
 */
@Service
@Slf4j
public class ChChannelRedEnvelopeHisServiceImpl implements ChChannelRedEnvelopeHisService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ChannelRedEnvelopeHisDAO channelRedEnvelopeHisDAO;

    @Reference(version = "1.0.0")
    private ChannelWalletService channelWalletService;

    @Reference(version = "1.0.0")
    private UserService userService;

    @Resource
    private ChAdminService chAdminService;

    @Resource
    private ChChannelNotifyHisService chChannelNotifyHisService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ChannelRedEnvelopeHisDO create(ChannelRedEnvelopeHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = channelRedEnvelopeHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ChannelRedEnvelopeHisDO> list) {
        boolean flag = channelRedEnvelopeHisDAO.insertBatch(list) > 0;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ChannelRedEnvelopeHisDO po) {
        boolean flag = channelRedEnvelopeHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ChannelRedEnvelopeHisDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = channelRedEnvelopeHisDAO.update(uw) > 0;
        return flag;
    }

    @Override
    public ChannelRedEnvelopeHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelRedEnvelopeHisDAO.selectOne(qw);
    }

    @Override
    public ChannelRedEnvelopeHisVO getVOById(Long id) {
        ChannelRedEnvelopeHisDO po = getById(id);
        if (po == null) {
            return null;
        }
        ChannelRedEnvelopeHisVO vo = new ChannelRedEnvelopeHisVO();
        BeanUtils.copyProperties(po, vo);
        return vo;
    }

    @Override
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelRedEnvelopeHisDAO.selectCount(qw) > 0;
    }

    @Override
    public PageList<ChannelRedEnvelopeHisDO> getPageList(ChRedEnvelopeHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ChannelRedEnvelopeHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(channelRedEnvelopeHisDAO.selectPageList(qw));
        page.setTotal(channelRedEnvelopeHisDAO.selectCount(qw));
        return page;
    }

    // 手动编码开始

    @Override
    public PageList<ChannelRedEnvelopeHisVO> getVOPageList(ChRedEnvelopeHisPageListQO qo) {
        PageList<ChannelRedEnvelopeHisVO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        PageList<ChannelRedEnvelopeHisDO> poPage = getPageList(qo);
        page.setTotal(poPage.getTotal());

        // 收红包的所有用户id
        Set<Long> userIds = new HashSet<>();
        Set<Long> adminIds = new HashSet<>();
        for (ChannelRedEnvelopeHisDO po : poPage.getList()) {
            List<ChannelExchangeOrder> orders = getOrders(po.getDetail());
            for (ChannelExchangeOrder order : orders) {
                userIds.add(order.getUserId());
            }
            adminIds.add(po.getAdminId());
        }
        Map<Long, UserDO> userMap = userService.getUserMapByIds(userIds);
        Map<Long, ChannelAdminDO> adminMap = chAdminService.getAdminMapByIds(adminIds);

        List<ChannelRedEnvelopeHisVO> voList = new ArrayList<>(poPage.getList().size());
        for (ChannelRedEnvelopeHisDO po : poPage.getList()) {
            ChannelRedEnvelopeHisVO vo = new ChannelRedEnvelopeHisVO();
            BeanUtils.copyProperties(po, vo);

            // 发红包明细
            List<ChannelExchangeOrder> orders = getOrders(po.getDetail());
            List<SendRedEnvelopeDetailVO> details = new ArrayList<>(orders.size());
            for (ChannelExchangeOrder order : orders) {
                SendRedEnvelopeDetailVO detailVO = new SendRedEnvelopeDetailVO();
                detailVO.setAmount(order.getAmount());
                UserDO user = userMap.get(order.getUserId());
                if (user != null) {
                    detailVO.setUserNo(user.getUserNo() + "");
                    detailVO.setUserName(user.getUserName());
                }
                details.add(detailVO);
            }
            vo.setSendType(getSendType(po.getAdminId()));
            vo.setDetails(details);

            ChannelAdminDO admin = adminMap.get(po.getAdminId());
            if (admin != null) {
                vo.setAdminName(admin.getAdminName());
                vo.setAdminAvatar(admin.getAdminAvatar());
            }

            voList.add(vo);
        }
        page.setList(voList);
        return page;
    }

    /**
     * 计算兑换总金额
     *
     * @param detailList
     * @return
     */
    @Override
    public Long getTotalAmount(List<RedEnvelopeDetail> detailList) {
        Long totalAmount = 0L;
        for (RedEnvelopeDetail detail : detailList) {
            totalAmount += detail.getAmount();
        }
        return totalAmount;
    }

    /**
     * 保存发送红包请求
     *
     * @param po
     * @param detailList
     * @return
     */
    @Override
    public ChannelRedEnvelopeHisDO saveRedEnvelope(ChannelRedEnvelopeHisDO po, List<RedEnvelopeDetail> detailList,
                                                   List<ChannelExchangeOrder> exchangeOrders) {
        po.setStatus(SendRedEnvStatusEnum.STORED.value());
        po.setTotalAmount(getTotalAmount(detailList));
        po.setDetail(JSON.toJSONString(exchangeOrders));

        ChannelRedEnvelopeHisDO poNew = create(po);
        if (poNew == null) {
            log.error("保存发送红包请求失败 {}", JSON.toJSONString(po));
            return null;
        }
        return poNew;
    }

    /**
     * 发红包（异步执行）
     *
     * @param channelId
     * @param hisId
     * @param exchangeOrders
     * @param tradeId        第三方交易id
     */
    @Override
    @Async
    public void asyncSendDiamond(Long channelId, Long hisId, List<ChannelExchangeOrder> exchangeOrders, String tradeId) {
        Long balance = sendDiamond(channelId, hisId, exchangeOrders);

        boolean isSuccess = false;
        if (balance != null && balance >= 0) {
            isSuccess = true;
        }

        RedEnvelopeNotifyRequest request = new RedEnvelopeNotifyRequest(tradeId, isSuccess);
        chChannelNotifyHisService.sendNotification(channelId, hisId, NotifyApiEnum.SEND_RED_ENVELOPE, request, 1);
    }

    /**
     * 发放星钻红包
     *
     * @param channelId
     * @param redEnvHisId    红包流水id
     * @param exchangeOrders
     * @return
     */
    @Override
    public Long sendDiamond(Long channelId, Long redEnvHisId, List<ChannelExchangeOrder> exchangeOrders) {
        ChannelRedEnvelopeHisDO po = getById(redEnvHisId);
        if (po == null) {
            log.error("无法发放星钻红包，没有对应的渠道发红包记录 hisId={}", redEnvHisId);
            return null;
        }
        Long balance = channelWalletService.exchangeDiamond(channelId, redEnvHisId, exchangeOrders);

        Integer status = SendRedEnvStatusEnum.FAIL.value();
        if (balance != null && balance >= 0) {
            status = SendRedEnvStatusEnum.SUCCESS.value();
        }

        String remark = null;
        if (balance == null) {
            remark = "发送红包失败";
        } else if (balance < 0) {
            remark = "发送红包失败，余额不足";
        }

        po = new ChannelRedEnvelopeHisDO();
        po.setHisId(redEnvHisId);
        po.setStatus(status);
        po.setRemark(remark);
        boolean flag = updateById(po);
        log.info("更新渠道发红包记录的处理状态 flag={} po={}", flag, po);

        return balance;
    }

    /**
     * 获得发送红包的订单列表
     *
     * @param detail
     * @return
     */
    private List<ChannelExchangeOrder> getOrders(String detail) {
        JSONArray array = JSON.parseArray(detail);
        if (array == null) {
            return Collections.EMPTY_LIST;
        }
        return JSON.parseArray(detail, ChannelExchangeOrder.class);
    }

    /**
     * 发送类型 1:管理员; 2:接口调用
     *
     * @param adminId
     * @return
     */
    private int getSendType(Long adminId) {
        return adminId > 0 ? 1 : 2;
    }
}