package com.starbuds.server.api.channel.service.admin.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.channel.common.pojo.db.ChannelSecretRsaDO;
import com.starbuds.server.api.channel.service.admin.dao.ChannelSecretRsaDAO;
import com.starbuds.server.api.channel.common.pojo.qo.ChSecretRsaPageListQO;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelSecretRsaVO;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.util.RsaPair;
import com.starbuds.server.common.utils.RsaUtil;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;

import org.springframework.beans.BeanUtils;

import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

import com.starbuds.server.api.channel.common.service.admin.ChChannelSecretRsaService;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Service;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 渠道RSA2信息服务
 */
@Service
@Slf4j
public class ChChannelSecretRsaServiceImpl implements ChChannelSecretRsaService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "chSecretRsaId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ChannelSecretRsaDAO channelSecretRsaDAO;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ChannelSecretRsaDO create(ChannelSecretRsaDO po) {
        po.setChSecretRsaId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = channelSecretRsaDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ChannelSecretRsaDO> list) {
        boolean flag = channelSecretRsaDAO.insertBatch(list) > 0;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ChannelSecretRsaDO po) {
        boolean flag = channelSecretRsaDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ChannelSecretRsaDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getChSecretRsaId());
        uw.setQw(qw);
        return update(uw);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = channelSecretRsaDAO.update(uw) > 0;
        return flag;
    }

    @Override
    public ChannelSecretRsaDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelSecretRsaDAO.selectOne(qw);
    }

    @Override
    public ChannelSecretRsaVO getVOById(Long id) {
        ChannelSecretRsaDO po = getById(id);
        if (po == null) {
            return null;
        }
        ChannelSecretRsaVO vo = new ChannelSecretRsaVO();
        BeanUtils.copyProperties(po, vo);
        return vo;
    }

    @Override
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelSecretRsaDAO.selectCount(qw) > 0;
    }

    @Override
    public PageList<ChannelSecretRsaDO> getPageList(ChSecretRsaPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ChannelSecretRsaDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(channelSecretRsaDAO.selectPageList(qw));
        page.setTotal(channelSecretRsaDAO.selectCount(qw));
        return page;
    }

    @Override
    public PageList<ChannelSecretRsaVO> getVOPageList(ChSecretRsaPageListQO qo) {
        PageList<ChannelSecretRsaVO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        PageList<ChannelSecretRsaDO> poPage = getPageList(qo);
        page.setTotal(poPage.getTotal());

        List<ChannelSecretRsaVO> voList = new ArrayList<>(poPage.getList().size());
        for (ChannelSecretRsaDO po : poPage.getList()) {
            ChannelSecretRsaVO vo = new ChannelSecretRsaVO();
            BeanUtils.copyProperties(po, vo);
            voList.add(vo);
        }
        page.setList(voList);
        return page;
    }

// 手动编码开始

    @Override
    public ChannelSecretRsaVO getRsaVO(Long channelId) {
        ChannelSecretRsaDO po = getRsa(channelId);
        if (po == null) {
            po = createRsa(channelId);
        }
        ChannelSecretRsaVO vo = new ChannelSecretRsaVO();
        BeanUtils.copyProperties(po, vo);
        return vo;
    }

    @Override
    public ChannelSecretRsaDO getRsa(Long channelId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("channelId", channelId);
        return channelSecretRsaDAO.selectOne(qw);
    }

    /**
     * 创建RSA
     *
     * @param channelId
     * @return
     */
    private ChannelSecretRsaDO createRsa(Long channelId) {
        ChannelSecretRsaDO po = new ChannelSecretRsaDO();
        po.setChannelId(channelId);
        po.setAdminId(0L);
        try {
            RsaPair pair = RsaUtil.generate();
            po.setXyPrivKey(pair.getPrivSt());
            po.setXyPubKey(pair.getPubStr());
        } catch (NoSuchAlgorithmException e) {
            log.info("生成RSA秘钥对失败", e);
            return null;
        }
        return create(po);
    }

}