package com.starbuds.server.api.channel.service.admin.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelAppVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelDTO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelInfoVO;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelSecretVO;
import com.starbuds.server.api.channel.common.service.admin.ChAdminService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelService;
import com.starbuds.server.api.channel.service.admin.dao.ChannelDAO;
import com.starbuds.server.api.channel.service.admin.dao.ChannelSecretDAO;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.ChannelDO;
import com.starbuds.server.common.pojo.db.ChannelSecretDO;
import com.starbuds.server.common.service.core.IdentityService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/4/28 14:42
 * @Description 渠道业务
 */
@Service
@Slf4j
public class ChChannelServiceImpl implements ChChannelService {

    @Resource
    ChannelDAO channelDAO;

    @Resource
    ChannelSecretDAO channelSecretDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    ChAdminService chAdminService;

    @Override
    public ChannelInfoVO getInfo(Long channelId) {
        return channelDAO.getInfo(channelId);
    }

    @Override
    public boolean existByChannelId(Long channelId) {
        return channelDAO.existByChannelId(channelId) > 0;
    }

    @Override
    public int update(ChannelDO channelDO) {
        return channelDAO.update(channelDO);
    }

    @Override
    public ChannelAppVO getChannelApp(Long channelId) {
        ChannelDO channelDO = channelDAO.selectChannelApp(channelId);
        if (channelDO == null) {
            return null;
        }

        ChannelAppVO vo = new ChannelAppVO();
        BeanUtils.copyProperties(channelDO, vo);
        String icon = channelDO.getAppIcon();
        if (StringUtils.isNotBlank(icon) && JSON.isValidArray(icon)) {
            vo.setAppIcon(JSON.parseArray(icon));
        }

        return vo;
    }

    @Override
    public int getSecretKeyCountByChannelId(Long channelId) {
        return channelSecretDAO.selectCountByChannelId(channelId);
    }

    @Override
    public int getActivatedCount(Long channelId) {
        return channelSecretDAO.selectActivatedCount(channelId);
    }

    @Override
    public String createSecretKey(long channelId) {
        String key = generateSecretKey();

        ChannelSecretDO secret = new ChannelSecretDO();
        secret.setSecretId(identityService.getId());
        secret.setChannelId(channelId);
        secret.setSecretKey(key);
        secret.setIsActivate(BoolIntVal.True);
        secret.setIsDeleted(BoolIntVal.False);
        secret.setCreateTime(System.currentTimeMillis());
        secret.setUpdateTime(secret.getCreateTime());

        int count = channelSecretDAO.insert(secret);

        if (count == 1) {
            channelSecretDAO.updateDisable(secret);
            log.info("创建秘钥成功 auth={}", secret);
            return key;
        }

        log.warn("创建秘钥失败 insertCount={} auth={}", count, secret);
        return null;
    }

    @Override
    public String changeSecretKey(Long channelId, Long secretId) {
        String key = generateSecretKey();
        ChannelSecretDO secret = new ChannelSecretDO();
        secret.setSecretKey(key);
        secret.setChannelId(channelId);
        secret.setSecretId(secretId);
        secret.setUpdateTime(System.currentTimeMillis());

        int count = channelSecretDAO.updateSecretKey(secret);
        if (count == 1) {
            log.info("更新秘钥成功 auth={}", secret);
            return key;
        }

        log.warn("更新秘钥失败 insertCount={} auth={}", count, secret);
        return null;
    }

    @Override
    public int deleteSecretKey(Long channelId, Long secretId) {
        ChannelSecretDO secret = new ChannelSecretDO();
        secret.setSecretId(secretId);
        secret.setChannelId(channelId);
        secret.setIsDeleted(BoolIntVal.True);
        secret.setUpdateTime(System.currentTimeMillis());
        return channelSecretDAO.updateByPkChannelId(secret);
    }

    @Override
    public int activateSecretKey(Long channelId, Long secretId, int isActivate) {
        ChannelSecretDO secret = new ChannelSecretDO();
        secret.setSecretId(secretId);
        secret.setChannelId(channelId);
        secret.setIsActivate(isActivate);
        secret.setUpdateTime(System.currentTimeMillis());
        channelSecretDAO.updateDisable(secret);
        return channelSecretDAO.updateByPkChannelId(secret);
    }

    @Override
    public List<ChannelSecretVO> getSecretKeyList(Long channelId) {
        return channelSecretDAO.selectList(channelId);
    }

    @Override
    public List<ChannelDTO> getGroupChannelList(Long channelGroupId) {
        if (channelGroupId == null) {
            return new ArrayList<>();
        }
        return channelDAO.selectListByChannelGroupId(channelGroupId);
    }

    @Override
    public PageList<ChannelInfoVO> getChannelPageList(Set<Long> channelIdSet, String channelName, Integer status, int pageIndex, int pageSize) {
        PageList<ChannelInfoVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(channelDAO.selectChannelPageList(channelIdSet, channelName, status, page.getPageCursor(), pageSize));
        page.setTotal(channelDAO.countChannelPageList(channelIdSet, channelName, status));
        return page;
    }

    @Override
    public List<ChannelDTO> getChannelList(Set<Long> authChannelIdSet) {
        return channelDAO.selectChannels(authChannelIdSet);
    }

    @Override
    public String getActivateSecretKey(Long channelId) {
        return null;
    }

    /**
     * 生成32位随机秘钥
     *
     * @return
     */
    private static String generateSecretKey() {
        String upperCaseLetters = RandomStringUtils.random(12, 65, 90, true, true);
        String lowerCaseLetters = RandomStringUtils.random(10, 97, 122, true, true);
        String numbers = RandomStringUtils.randomNumeric(10);
        String combinedChars = upperCaseLetters.concat(lowerCaseLetters)
                .concat(numbers);
        List<Character> pwdChars = combinedChars.chars()
                .mapToObj(c -> (char) c)
                .collect(Collectors.toList());
        Collections.shuffle(pwdChars);
        String password = pwdChars.stream()
                .collect(StringBuilder::new, StringBuilder::append, StringBuilder::append)
                .toString();
        return password;
    }
}
