package com.starbuds.server.api.channel.service.admin.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.channel.service.admin.dao.ChannelWalletHisDAO;
import com.starbuds.server.api.channel.common.pojo.qo.ChWalletHisPageListQO;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelWalletHisVO;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.ChannelWalletHisDO;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;

import org.springframework.beans.BeanUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import com.starbuds.server.api.channel.common.service.admin.ChChannelWalletHisService;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Service;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 渠道钱包变更记录服务
 */
@Service
@Slf4j
public class ChChannelWalletHisServiceImpl implements ChChannelWalletHisService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "hisId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private ChannelWalletHisDAO channelWalletHisDAO;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ChannelWalletHisDO create(ChannelWalletHisDO po) {
        po.setHisId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = channelWalletHisDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("创建成功 {}", json);
            return po;
        }
        log.warn("创建失败 {}", json);
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<ChannelWalletHisDO> list) {
        boolean flag = channelWalletHisDAO.insertBatch(list) > 0;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(ChannelWalletHisDO po) {
        boolean flag = channelWalletHisDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(ChannelWalletHisDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getHisId());
        uw.setQw(qw);
        return update(uw);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = channelWalletHisDAO.update(uw) > 0;
        return flag;
    }

    @Override
    public ChannelWalletHisDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelWalletHisDAO.selectOne(qw);
    }

    @Override
    public ChannelWalletHisVO getVOById(Long id) {
        ChannelWalletHisDO po = getById(id);
        if (po == null) {
            return null;
        }
        ChannelWalletHisVO vo = new ChannelWalletHisVO();
        BeanUtils.copyProperties(po, vo);
        return vo;
    }

    @Override
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return channelWalletHisDAO.selectCount(qw) > 0;
    }

    @Override
    public PageList<ChannelWalletHisDO> getPageList(ChWalletHisPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ChannelWalletHisDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(channelWalletHisDAO.selectPageList(qw));
        page.setTotal(channelWalletHisDAO.selectCount(qw));
        return page;
    }

    @Override
    public PageList<ChannelWalletHisVO> getVOPageList(ChWalletHisPageListQO qo) {
        PageList<ChannelWalletHisVO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        PageList<ChannelWalletHisDO> poPage = getPageList(qo);
        page.setTotal(poPage.getTotal());

        List<ChannelWalletHisVO> voList = new ArrayList<>(poPage.getList().size());
        for (ChannelWalletHisDO po : poPage.getList()) {
            ChannelWalletHisVO vo = new ChannelWalletHisVO();
            BeanUtils.copyProperties(po, vo);
            voList.add(vo);
        }
        page.setList(voList);
        return page;
    }

// 手动编码开始

}