package com.starbuds.server.api.channel.service.admin.impl;

import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.channel.common.pojo.constant.SendRedEnvStatusEnum;
import com.starbuds.server.api.channel.common.pojo.qo.ChWalletPageListQO;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelWalletVO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelRedEnvelopeHisService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelWalletService;
import com.starbuds.server.api.channel.service.admin.dao.ChannelWalletDAO;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.db.ChannelRedEnvelopeHisDO;
import com.starbuds.server.common.pojo.db.ChannelWalletDO;
import com.starbuds.server.common.pojo.finance.ChannelExchangeOrder;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.finance.ChannelWalletService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

/**
 * 渠道钱包服务
 */
@Service
@Slf4j
public class ChChannelWalletServiceImpl implements ChChannelWalletService {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "chWalletId";

    @Resource
    private ChannelWalletDAO channelWalletDAO;

    @Resource
    private ChChannelRedEnvelopeHisService chChannelRedEnvelopeHisService;

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Override
    public PageList<ChannelWalletDO> getPageList(ChWalletPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<ChannelWalletDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(channelWalletDAO.selectPageList(qw));
        page.setTotal(channelWalletDAO.selectCount(qw));
        return page;
    }

    @Override
    public PageList<ChannelWalletVO> getVOPageList(ChWalletPageListQO qo) {
        PageList<ChannelWalletVO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        PageList<ChannelWalletDO> poPage = getPageList(qo);
        page.setTotal(poPage.getTotal());

        List<ChannelWalletVO> voList = new ArrayList<>(poPage.getList().size());
        for (ChannelWalletDO po : poPage.getList()) {
            ChannelWalletVO vo = new ChannelWalletVO();
            BeanUtils.copyProperties(po, vo);
            voList.add(vo);
        }
        page.setList(voList);
        return page;
    }

// 手动编码开始

    /**
     * 查询渠道钱包
     *
     * @param channelId
     * @return
     */
    @Override
    public ChannelWalletVO getWalletVO(Long channelId) {
        ChannelWalletDO po = getWallet(channelId);
        if (po == null) {
            return null;
        }
        ChannelWalletVO vo = new ChannelWalletVO();
        BeanUtils.copyProperties(po, vo);
        return vo;
    }

    /**
     * 发星钻红包
     *
     * @param userId
     * @param amount
     * @param adminId
     * @param ip
     * @return
     */
    @Override
    public Long sendDiamond(Long channelId, Long userId, Long amount, Long adminId, String ip) {
        ChannelExchangeOrder order = new ChannelExchangeOrder();
        order.setUserId(userId);
        order.setAmount(amount);
        List<ChannelExchangeOrder> exchangeOrders = new ArrayList<>(1);
        exchangeOrders.add(order);

        ChannelRedEnvelopeHisDO po = new ChannelRedEnvelopeHisDO();
        po.setChannelId(channelId);
        po.setTradeId("" + identityService.getId());
        po.setCoinType(WalletTypeEnum.STAR_DIAMOND.getValue());
        po.setTotalAmount(amount);
        po.setDetail(JSON.toJSONString(exchangeOrders));
        po.setStatus(SendRedEnvStatusEnum.STORED.value());
        po.setIp(ip);
        po.setAdminId(adminId);

        po = chChannelRedEnvelopeHisService.create(po);
        if (po == null) {
            log.error("发星钻红包失败");
            return null;
        }

        return chChannelRedEnvelopeHisService.sendDiamond(channelId, po.getHisId(), exchangeOrders);
    }

    private ChannelWalletDO getWallet(long channelId) {
        String walletId = getWalletId(channelId, WalletTypeEnum.STAR_DIAMOND.getValue());
        QueryWrapper qw = new QueryWrapper();
        qw.eq("walletId", walletId);
        return channelWalletDAO.selectOne(qw);
    }

    private String getWalletId(long channelId, int type) {
        return SecureUtil.md5(String.format("%d:%d", channelId, type));
    }

}