package com.starbuds.server.api.channel.service.admin.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.api.channel.common.pojo.admin.*;
import com.starbuds.server.api.channel.common.pojo.constant.StatTagTypeEnum;
import com.starbuds.server.api.channel.common.service.admin.ChStatPlatformService;
import com.starbuds.server.api.channel.service.admin.dao.StatChannelDAO;
import com.starbuds.server.api.channel.service.admin.dao.StatPlatformDAO;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.constant.statistics.TimeTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.service.user.ChannelService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/2/18 15:36
 * @Description 平台统计业务实现
 */
@Slf4j
@Service
public class ChStatPlatformServiceImpl implements ChStatPlatformService {

    @Resource
    StatPlatformDAO statPlatformDAO;

    @Resource
    StatChannelDAO statChannelDAO;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Override
    public PageList<StatPlatformVO> getStatPlatformPageList(Set<Long> channelIds,
                                                            Integer statType,
                                                            Integer statTimeType,
                                                            int pageIndex,
                                                            int pageSize) {
        PageList<StatPlatformVO> page = new PageList<>(pageIndex, pageSize);
        List<StatPlatformVO> pageList = statPlatformDAO.getStatPlatformPageList(channelIds, statType, statTimeType, page.getPageCursor(), pageSize);
        if (!CollectionUtils.isEmpty(pageList)) {
            for (StatPlatformVO vo : pageList) {
                vo.setChannelName(channelService.getChannelName(vo.getChannelId()));
                vo.setStatName(StatPlatformTypeEnum.toEnum(vo.getStatType()).getDesc());
            }
        }
        page.setList(pageList);
        page.setTotal(statPlatformDAO.countStatPlatformPageList(channelIds, statType, statTimeType));
        return page;
    }

    @Override
    public List<CurrentAndYesterdayStatEntity> getCurrentAndYesterdayStat(Long channelId) {
        List<CurrentAndYesterdayStatEntity> statEntities = new ArrayList<>();
        List<Long> dateList = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Calendar instance = Calendar.getInstance();
        Long today = Convert.toLong(sdf.format(instance.getTime()));
        dateList.add(today);
        instance.add(Calendar.DAY_OF_YEAR, -1);
        Long yesterday = Convert.toLong(sdf.format(instance.getTime()));
        dateList.add(yesterday);
        // 总数据
        List<StatPlatformVO> allStats = statPlatformDAO.selectByStatTime(dateList, channelId);
        if (!CollectionUtils.isEmpty(allStats)) {
            // 根据日期分组
            Map<Long, List<StatPlatformVO>> groupByTime = allStats.stream().collect(Collectors.groupingBy(StatPlatformVO::getStatTime));
            // 今天统计
            List<StatPlatformVO> todayStats = groupByTime.get(today);
            // 昨天统计
            List<StatPlatformVO> yesterdayStats = groupByTime.get(yesterday);
            // 遍历统计类型
            for (StatPlatformTypeEnum item : StatPlatformTypeEnum.values()) {
                CurrentAndYesterdayStatEntity statEntity = new CurrentAndYesterdayStatEntity();
                statEntity.setCurrent(today);
                statEntity.setYesterday(yesterday);
                statEntity.setCurrentValue(todayStats == null ? BigDecimal.ZERO : Optional.ofNullable(todayStats.stream().filter(e -> e.getStatType() == item.getValue() && e.getStatTime().equals(today)).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add)).orElse(BigDecimal.ZERO));
                statEntity.setYesterdayValue(yesterdayStats == null ? BigDecimal.ZERO : Optional.ofNullable(yesterdayStats.stream().filter(e -> e.getStatType() == item.getValue() && e.getStatTime().equals(yesterday)).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add)).orElse(BigDecimal.ZERO));
                statEntity.setDiffValue(statEntity.getCurrentValue().subtract(statEntity.getYesterdayValue()));
                if (BigDecimal.ZERO.compareTo(statEntity.getCurrentValue()) == 0) {
                    statEntity.setGrowthPercent(BigDecimal.ZERO);
                } else if (BigDecimal.ZERO.compareTo(statEntity.getYesterdayValue()) == 0) {
                    statEntity.setGrowthPercent(BigDecimal.ONE);
                } else {
                    statEntity.setGrowthPercent(statEntity.getDiffValue().divide(statEntity.getYesterdayValue(), 4, RoundingMode.HALF_UP));
                }
                statEntity.setStatType(item.getValue());
                statEntity.setStatName(item.getDesc());
                statEntities.add(statEntity);
            }
        }
        return statEntities;
    }

    @Override
    public StatDetail getStatDetail(Long channelId, Integer timeType, Long startTime, Long endTime, Integer statType) {
        StatDetail statDetail = new StatDetail();
        try {
            // 格式化时间
            String requirePattern = "";
            SimpleDateFormat sdf = null;
            Calendar tempStart = null;
            Calendar tempEnd = null;
            // 时间类型
            TimeTypeEnum typeEnum = TimeTypeEnum.toEnum(timeType);
            // 时间区间
            List<Long> dateRang = new ArrayList<>();
            switch (typeEnum) {
                case YEAR:
                    requirePattern = "yyyyMM";
                    sdf = new SimpleDateFormat(requirePattern);
                    tempStart = Calendar.getInstance();
                    tempStart.set(Calendar.YEAR, Convert.toInt(startTime));
                    tempStart.set(Calendar.MONTH, 0);
                    tempEnd = Calendar.getInstance();
                    tempEnd.set(Calendar.YEAR, Convert.toInt(endTime));
                    tempEnd.set(Calendar.MONTH, 11);
                    for (int i = 0; i < 12; i++) {
                        dateRang.add(Convert.toLong(sdf.format(tempStart.getTime())));
                        tempStart.add(Calendar.MONTH, 1);
                    }
                    break;
                case MONTH:
                    requirePattern = "yyyyMMdd";
                    sdf = new SimpleDateFormat(requirePattern);
                    tempStart = Calendar.getInstance();
                    tempStart.setTime(DateUtil.parse(startTime.toString(), "yyyyMM"));
                    tempEnd = Calendar.getInstance();
                    tempEnd.setTime(DateUtil.parse(endTime.toString(), "yyyyMM"));
                    tempEnd.set(Calendar.DATE, tempEnd.getActualMaximum(Calendar.DATE));//当月最后一天
                    while (tempStart.compareTo(tempEnd) <= 0) {
                        dateRang.add(Convert.toLong(sdf.format(tempStart.getTime())));
                        tempStart.add(Calendar.DAY_OF_MONTH, 1);
                    }
                    break;
                case DAY:
                    requirePattern = "yyyyMMddHH";
                    sdf = new SimpleDateFormat(requirePattern);
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(DateUtil.parse(startTime.toString(), "yyyyMMdd"));
                    // 所有时间段
                    for (int i = 0; i < 24; i++) {
                        dateRang.add(Convert.toLong(sdf.format(calendar.getTime())));
                        calendar.add(Calendar.HOUR, 1);
                    }
                    break;
                case WEEK:
                    requirePattern = "yyyyMMdd";
                    sdf = new SimpleDateFormat(requirePattern);
                    tempStart = Calendar.getInstance();
                    tempStart.setTime(sdf.parse(startTime.toString()));
                    tempEnd = Calendar.getInstance();
                    tempEnd.setTime(sdf.parse(endTime.toString()));
                    while (tempStart.compareTo(tempEnd) <= 0) {
                        dateRang.add(Convert.toLong(sdf.format(tempStart.getTime())));
                        tempStart.add(Calendar.DAY_OF_YEAR, 1);
                    }
                    break;
                default:
                    break;
            }
            // 区间的时间集合
            log.info("时间格式：" + requirePattern);
            log.info("区间时间：" + dateRang);
            // 从db查询的区间数据
            List<StatChannelDetail> statDetails = statPlatformDAO.selectStatDetail(channelId, statType, dateRang.get(0), dateRang.get(dateRang.size() - 1));

            // 总统计
            List<StatTotalDetail> totalStat = new ArrayList<>();
            // 渠道统计
            List<StatChannelDetail> channelStat = new ArrayList<>();
            if (!CollectionUtils.isEmpty(statDetails)) {
                // 总统计按时间分组
                Map<Long, List<StatChannelDetail>> groupByTime = statDetails.stream().collect(Collectors.groupingBy(StatChannelDetail::getStatTime));
                for (Map.Entry<Long, List<StatChannelDetail>> entry : groupByTime.entrySet()) {
                    StatTotalDetail totalItem = new StatTotalDetail();
                    totalItem.setStatTime(entry.getKey());
                    totalItem.setStatValue(entry.getValue().stream().map(StatChannelDetail::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                    totalStat.add(totalItem);
                }

                // 总渠道里没有时间
                List<Long> remainDate = new ArrayList<>(dateRang);
                remainDate.removeAll(totalStat.stream().map(StatTotalDetail::getStatTime).collect(Collectors.toList()));
                if (!CollectionUtils.isEmpty(remainDate)) {
                    for (Long item : remainDate) {
                        StatTotalDetail totalItem = new StatTotalDetail();
                        totalItem.setStatTime(item);
                        totalItem.setStatValue(BigDecimal.ZERO);
                        totalStat.add(totalItem);
                    }
                }

                // 按照渠道分组
                Map<Long, List<StatChannelDetail>> groupByChannel = statDetails.stream().collect(Collectors.groupingBy(StatChannelDetail::getChannelId));
                for (Map.Entry<Long, List<StatChannelDetail>> entry : groupByChannel.entrySet()) {
                    Long mapKey = entry.getKey();
                    List<StatChannelDetail> mapValue = entry.getValue();
                    for (Long date : dateRang) {
                        StatChannelDetail channelDetail = new StatChannelDetail();
                        channelDetail.setChannelId(mapKey);
                        channelDetail.setStatTime(date);
                        channelDetail.setChannelName(channelService.getChannelName(entry.getKey()));
                        // 相同渠道下的
                        channelDetail.setStatValue(mapValue.stream().filter(e -> mapKey.equals(e.getChannelId()) && date.equals(e.getStatTime())).map(StatChannelDetail::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                        channelStat.add(channelDetail);
                    }
                }
            } else {
                if (!CollectionUtils.isEmpty(dateRang)) {
                    for (Long date : dateRang) {
                        StatTotalDetail totalItem = new StatTotalDetail();
                        totalItem.setStatTime(date);
                        totalItem.setStatValue(BigDecimal.ZERO);
                        totalStat.add(totalItem);
                    }
                }
            }

            statDetail.setTotalStat(totalStat.stream().sorted(Comparator.comparing(StatTotalDetail::getStatTime)).collect(Collectors.toList()));
            statDetail.setChannelStat(channelStat.stream().sorted(Comparator.comparing(StatChannelDetail::getStatTime)).collect(Collectors.toList()));
            statDetail.setTimeType(timeType);
            statDetail.setTimeName(typeEnum.getDesc());
            statDetail.setStartTime(dateRang.get(0));
            statDetail.setEndTime(dateRang.get(dateRang.size() - 1));
            statDetail.setStatType(statType);
            statDetail.setStatName(StatPlatformTypeEnum.toEnum(statType).getDesc());
        } catch (Exception e) {
            log.info("获取统计出错{}", e);
        }
        return statDetail;
    }

    @Override
    public List<StatPeriodOverViewTop> getPeriodOverviewTop(String channelIds, Long startDate, Long endDate) {

        List<StatPeriodOverViewTop> viewTops = new ArrayList<>();
        // 统计类型
        List<Integer> statTypes = Arrays.asList(new Integer[]{
                StatPlatformTypeEnum.DOWNLOAD_COUNT.getValue(),
                StatPlatformTypeEnum.INSTALL_COUNT.getValue(),
                StatPlatformTypeEnum.REGISTER_USERS.getValue(),
                StatPlatformTypeEnum.ACTIVE_USERS.getValue(),
                StatPlatformTypeEnum.RECHARGE_AMOUNT.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_AMOUNT.getValue(),
                StatPlatformTypeEnum.RECHARGE_USERS.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_USERS.getValue(),
                StatPlatformTypeEnum.JOIN_RTC_ROOM_USERS.getValue(),
        });

        List<Long> channelIdList = null;
        if (StringUtils.isNotEmpty(channelIds)) {
            channelIdList = Arrays.stream(channelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toList());
        }
        List<StatPlatformVO> topStat = statPlatformDAO.selectStat(channelIdList, statTypes, StatTimeType.YEAR_MONTH_DAY, startDate, endDate);
        for (Integer statType : statTypes) {
            StatPeriodOverViewTop top = new StatPeriodOverViewTop();
            top.setStatType(statType);
            top.setStatName(StatPlatformTypeEnum.toEnum(statType).getDesc());
            if (!CollectionUtil.isEmpty(topStat)) {
                Map<Integer, List<StatPlatformVO>> groupByStatType = topStat.stream().collect(Collectors.groupingBy(StatPlatformVO::getStatType));
                if (!CollectionUtils.isEmpty(groupByStatType)) {
                    for (Map.Entry<Integer, List<StatPlatformVO>> entry : groupByStatType.entrySet()) {
                        if (statType.equals(entry.getKey())) {
                            top.setStatValue(entry.getValue() == null ? BigDecimal.ZERO : entry.getValue().stream().filter(e -> statType.equals(e.getStatType())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                        }
                    }
                }
            }
            viewTops.add(top);
        }
        return viewTops;
    }

    @Override
    public StatPeriodOverViewMiddle getPeriodOverviewMiddle(String channelIdArray, Long startDate, Long endDate, Integer tagType) {

        StatTagTypeEnum statTag = StatTagTypeEnum.toEnum(tagType);

        Set<Long> datetimes = new HashSet<>();
        Set<Long> todayDatetimes = new HashSet<>();
        Set<Long> yesterdayDatetimes = new HashSet<>();
        Set<Long> oneWeekAgoDatetimes = new HashSet<>();
        Set<Long> searchDatetimes = new HashSet<>();

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Calendar instance = Calendar.getInstance();
        Long today = Convert.toLong(sdf.format(instance.getTime()));
        // 日期范围
        List<Long> dateRangs = getDays(startDate, endDate);
        instance.add(Calendar.DAY_OF_YEAR, -1);
        Long yesterday = Convert.toLong(sdf.format(instance.getTime()));
        instance.add(Calendar.DAY_OF_YEAR, -6);
        Long oneWeekAgo = Convert.toLong(sdf.format(instance.getTime()));
        // 小时集合
        List<String> hours = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            String hour = String.format("%02d", i);
            hours.add(hour);
            todayDatetimes.add(Convert.toLong(today + hour));
            yesterdayDatetimes.add(Convert.toLong(yesterday + hour));
            oneWeekAgoDatetimes.add(Convert.toLong(oneWeekAgo + hour));
            if (!CollectionUtils.isEmpty(dateRangs)) {
                for (Long item : dateRangs) {
                    searchDatetimes.add(Convert.toLong(item + hour));
                }
            }
        }
        List<Long> dateList = new ArrayList<>();
        dateList.add(today);
        dateList.add(yesterday);
        dateList.add(oneWeekAgo);
        datetimes.addAll(todayDatetimes);
        datetimes.addAll(yesterdayDatetimes);
        datetimes.addAll(oneWeekAgoDatetimes);
        // 渠道
        List<Long> channelIdList = null;
        if (StringUtils.isNotEmpty(channelIdArray)) {
            channelIdList = Arrays.stream(channelIdArray.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toList());
        }
        // 统计类型
        List<Integer> statTypes = getStatTypes(statTag);
        if (CollectionUtils.isEmpty(statTypes)) {
            log.info("暂无类型统计数据");
            return null;
        }
        return getViewMiddle(statTag, hours, statTypes, datetimes, dateList, channelIdList, searchDatetimes, dateRangs);
    }

    /**
     * 获取中间视图结果
     *
     * @param statTag
     * @param hours
     * @param statTypes
     * @param datetimes
     * @param dateList
     * @param channelIdList
     * @param rangDatetimes
     * @param dateRangs
     * @return
     */
    private StatPeriodOverViewMiddle getViewMiddle(StatTagTypeEnum statTag, List<String> hours, List<Integer> statTypes, Set<Long> datetimes, List<Long> dateList, List<Long> channelIdList, Set<Long> rangDatetimes, List<Long> dateRangs) {
        StatPeriodOverViewMiddle viewMiddle = new StatPeriodOverViewMiddle();
        viewMiddle.setTagType(statTag.getCode());
        viewMiddle.setTagName(statTag.getDesc());
        List<StatPlatformVO> middleStat = statPlatformDAO.selectStatForPeriodOverviewMiddle(channelIdList, statTypes, StatTimeType.YEAR_MONTH_DAY_HOUR, datetimes);
        int i = 0;
        for (Long date : dateList) {
            // 同一天的数据
            List<StatPeriodOverViewMiddle.Data> middleDatas = new ArrayList<>();
            for (String hour : hours) {
                StatPeriodOverViewMiddle.Data data = new StatPeriodOverViewMiddle.Data();
                data.setHourStr(hour);
                data.setDateStr(Convert.toStr(date));
                data.setStatTime(date + hour);
                Long datetime = Convert.toLong(data.getStatTime());
                List<StatPlatformVO> filterByDatetime = middleStat.stream().filter(e -> datetime.equals(Convert.toLong(Convert.toStr(e.getStatTime())))).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(filterByDatetime)) {
                    for (StatPlatformVO vo : filterByDatetime) {
                        // 日期小时相同
                        if (vo.getStatTime().equals(datetime)) {
                            // 查询统计类型只有一个，即是计算结果
                            if (statTypes.size() == 1) {
                                data.setValue1(filterByDatetime.stream().filter(e -> statTypes.get(0).equals(e.getStatType()) && hour.equals(Convert.toStr(datetime).substring(Convert.toStr(datetime).length() - 2)) && datetime.equals(e.getStatTime())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                            } else if (statTypes.size() > 1) {
                                // 第一个类型的值
                                data.setValue1(filterByDatetime.stream().filter(e -> statTypes.get(0).equals(e.getStatType()) && hour.equals(Convert.toStr(datetime).substring(Convert.toStr(datetime).length() - 2)) && date.equals(Convert.toLong(Convert.toStr(e.getStatTime()).substring(0, 8)))).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                                // 第二个类型的值
                                data.setValue2(filterByDatetime.stream().filter(e -> statTypes.get(1).equals(e.getStatType()) && hour.equals(Convert.toStr(datetime).substring(Convert.toStr(datetime).length() - 2)) && date.equals(Convert.toLong(Convert.toStr(e.getStatTime()).substring(0, 8)))).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                            }
                            data.setResult(caclResult(data.getValue1(), data.getValue2()));
                        }
                    }
                }
                middleDatas.add(data);
            }
            if (i == 0) {
                viewMiddle.setTodayData(middleDatas);
            } else if (i == 1) {
                viewMiddle.setYesterdayData(middleDatas);
            } else if (i == 2) {
                viewMiddle.setOneWeekAgoData(middleDatas);
            }
            i++;
        }

        // 说明此处为区间的值
        if (!CollectionUtils.isEmpty(rangDatetimes)) {
            // 区间的数据
            middleStat = statPlatformDAO.selectStatForPeriodOverviewMiddle(channelIdList, statTypes, StatTimeType.YEAR_MONTH_DAY_HOUR, rangDatetimes);
            List<StatPeriodOverViewMiddle.Data> middleDatas = new ArrayList<>();
            for (String hour : hours) {
                StatPeriodOverViewMiddle.Data data = new StatPeriodOverViewMiddle.Data();
                data.setHourStr(hour);
                for (Long datetime : datetimes) {
                    // 小时相同
                    if (hour.equals(Convert.toStr(datetime).substring(Convert.toStr(datetime).length() - 2))) {
                        data.setDateStr(String.join(",", dateRangs.stream().map(e -> Convert.toStr(e)).collect(Collectors.toList())));
                        data.setStatTime(String.join(",", dateRangs.stream().map(e -> e + hour).collect(Collectors.toList())));
                    }
                    if (!CollectionUtils.isEmpty(middleStat)) {
                        for (StatPlatformVO vo : middleStat) {
                            // 日期小时相同
                            if (datetime.equals(vo.getStatTime())) {
                                // 查询统计类型只有一个，即是计算结果
                                if (statTypes.size() == 1) {
                                    data.setValue1((middleStat.stream().filter(e -> statTypes.get(0).equals(e.getStatType()) && hour.equals(Convert.toStr(datetime).substring(Convert.toStr(datetime).length() - 2)) && datetime.equals(e.getStatTime())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add)));
                                } else if (statTypes.size() > 1) {
                                    // 第一个类型的值
                                    data.setValue1(middleStat.stream().filter(e -> statTypes.get(0).equals(e.getStatType()) && hour.equals(Convert.toStr(datetime).substring(Convert.toStr(datetime).length() - 2)) && datetime.equals(e.getStatTime())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                                    // 第二个类型的值
                                    data.setValue2(middleStat.stream().filter(e -> statTypes.get(1).equals(e.getStatType()) && hour.equals(Convert.toStr(datetime).substring(Convert.toStr(datetime).length() - 2)) && datetime.equals(e.getStatTime())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                                }
                                data.setResult(caclResult(data.getValue1(), data.getValue2()));
                            }
                        }
                    }
                }
                middleDatas.add(data);
            }
            viewMiddle.setRangData(middleDatas);
            viewMiddle.setStatType1(statTypes.get(0));
            viewMiddle.setStatTypeName1(StatPlatformTypeEnum.toEnum(viewMiddle.getStatType1()).getDesc());
            if (statTypes.size() > 1) {
                viewMiddle.setStatType2(statTypes.get(1));
                viewMiddle.setStatTypeName2(StatPlatformTypeEnum.toEnum(viewMiddle.getStatType2()).getDesc());
            }
        }

        return viewMiddle;
    }

    /**
     * 计算结果
     * 分三种情况：分子为0、分母为0和正常计算结果
     *
     * @param value1
     * @param value2
     * @return
     */
    private BigDecimal caclResult(BigDecimal value1, BigDecimal value2) {
        if ((value1 == null && value2 == null) || value1.compareTo(value2) == 0 && value1.compareTo(BigDecimal.ZERO) == 0) {
            return BigDecimal.ZERO;
        } else if (value1.compareTo(value2) != 0 && value2.compareTo(BigDecimal.ZERO) == 0) {
            return value1;
        }
        // 运算得出结果，如果分子为0、分母为0和正常计算结果
        if (value1.compareTo(BigDecimal.ZERO) == 0) {
            return BigDecimal.ZERO;
        } else if (value2.compareTo(BigDecimal.ZERO) == 0) {
            return BigDecimal.ONE.setScale(4, RoundingMode.HALF_UP);
        } else {
            return value1.divide(value2, 4, RoundingMode.HALF_UP);
        }
    }

    @Override
    public Map<String, List<StatPeriodOverViewBottom>> getPeriodOverviewBottom(String channelIdArray, Long startDate, Long endDate) {
        // 统计类型
        List<Integer> statTypes = Arrays.asList(new Integer[]{
                StatPlatformTypeEnum.DOWNLOAD_COUNT.getValue(),
                StatPlatformTypeEnum.INSTALL_COUNT.getValue(),
                StatPlatformTypeEnum.REGISTER_USERS.getValue(),
                StatPlatformTypeEnum.ACTIVE_USERS.getValue(),
                StatPlatformTypeEnum.JOIN_ROOM_USERS.getValue(),
                StatPlatformTypeEnum.SINGLE_CHAT_TIMES_CONNECT.getValue(),
                StatPlatformTypeEnum.RECHARGE_AMOUNT.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_AMOUNT.getValue(),
                StatPlatformTypeEnum.RECHARGE_USERS.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_USERS.getValue(),
                StatPlatformTypeEnum.LIVE_ANCHORS.getValue(),
                StatPlatformTypeEnum.JOIN_RTC_ROOM_USERS.getValue(),
        });

        String startDateStr = Convert.toStr(startDate) + "00";
        String endDateStr = Convert.toStr(endDate) + "23";

        List<Long> channelIdList = null;
        if (StringUtils.isNotEmpty(channelIdArray)) {
            channelIdList = Arrays.stream(channelIdArray.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toList());
        }
        List<StatPlatformVO> topStat = statPlatformDAO.selectStat(channelIdList, statTypes, StatTimeType.YEAR_MONTH_DAY_HOUR, Convert.toLong(startDateStr), Convert.toLong(endDateStr));

        SimpleDateFormat hourFormat = new SimpleDateFormat("HH");
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
        String curHour = hourFormat.format(new Date());
        String curDate = dateFormat.format(new Date());
        // 小时集合
        List<String> hours = new ArrayList<>();
        int forI = Convert.toLong(curDate).equals(startDate) ? Convert.toInt(curHour) : 24;
        for (int i = 0; i < forI; i++) {
            hours.add(String.format("%02d", i));
        }
        Map<String, List<StatPeriodOverViewBottom>> map = new LinkedHashMap<>();
        for (String hour : hours) {
            List<StatPeriodOverViewBottom> bottoms = new ArrayList<>();
            for (Integer statType : statTypes) {
                StatPeriodOverViewBottom bottom = new StatPeriodOverViewBottom();
                bottom.setStatType(statType);
                bottom.setStatName(StatPlatformTypeEnum.toEnum(statType).getDesc());
                if (!CollectionUtil.isEmpty(topStat)) {
                    // 相同类型
                    Map<Integer, List<StatPlatformVO>> groupByStatType = topStat.stream().collect(Collectors.groupingBy(StatPlatformVO::getStatType));
                    if (!CollectionUtils.isEmpty(groupByStatType)) {
                        for (Map.Entry<Integer, List<StatPlatformVO>> entry : groupByStatType.entrySet()) {
                            // 相同类型，相同小时
                            if (statType.equals(entry.getKey())) {
                                bottom.setStatValue(entry.getValue() == null ? BigDecimal.ZERO : entry.getValue().stream().filter(e -> statType.equals(e.getStatType()) && hour.equals(Convert.toStr(e.getStatTime()).substring(Convert.toStr(e.getStatTime()).length() - 2))).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                            }
                        }
                    }
                }
                bottoms.add(bottom);
            }
            map.put(hour, bottoms);
        }
        return map;
    }

    @Override
    public List<OverallTrendTop> getOverallTrendTop(Set<Long> channelIdSet, Long startDate, Long endDate) {

        // 统计类型
        List<Integer> statTypes = Arrays.asList(new Integer[]{
                StatPlatformTypeEnum.DOWNLOAD_COUNT.getValue(),
                StatPlatformTypeEnum.INSTALL_COUNT.getValue(),
                StatPlatformTypeEnum.REGISTER_USERS.getValue(),
                StatPlatformTypeEnum.JOIN_ROOM_USERS.getValue(),
                StatPlatformTypeEnum.JOIN_RTC_ROOM_USERS.getValue(),
                StatPlatformTypeEnum.SINGLE_CHAT_TIMES_CONNECT.getValue(),
                StatPlatformTypeEnum.ACTIVE_USERS.getValue(),
                StatPlatformTypeEnum.LIVE_ANCHORS.getValue(),
                StatPlatformTypeEnum.RECHARGE_AMOUNT.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_AMOUNT.getValue(),
                StatPlatformTypeEnum.RECHARGE_USERS.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_USERS.getValue(),
        });

        List<StatPlatformVO> topStat = statChannelDAO.selectStat(channelIdSet, statTypes, StatTimeType.YEAR_MONTH_DAY, startDate, endDate);

        List<OverallTrendTop> tops = new ArrayList<>();
        for (Integer statType : statTypes) {
            OverallTrendTop top = new OverallTrendTop();
            top.setStatType(statType);
            top.setStatName(StatPlatformTypeEnum.toEnum(statType).getDesc());
            if (!CollectionUtil.isEmpty(topStat)) {
                Map<Integer, List<StatPlatformVO>> groupByStatType = topStat.stream().collect(Collectors.groupingBy(StatPlatformVO::getStatType));
                if (!CollectionUtils.isEmpty(groupByStatType)) {
                    for (Map.Entry<Integer, List<StatPlatformVO>> entry : groupByStatType.entrySet()) {
                        if (statType.equals(entry.getKey())) {
                            top.setStatValue(entry.getValue() == null ? BigDecimal.ZERO : entry.getValue().stream().filter(e -> statType.equals(e.getStatType())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                        }
                    }
                }
            }
            tops.add(top);
        }
        return tops;
    }

    @Override
    public OverallTrendMiddle getOverallTrendMiddle(String channelIdArray, Long startDate, Long endDate, Integer tagType) {
        OverallTrendMiddle middle = new OverallTrendMiddle();

        StatTagTypeEnum statTag = StatTagTypeEnum.toEnum(tagType);
        // 日期
        List<Long> dataRangs = getDays(startDate, endDate);
        // 渠道id
        List<Long> channelIdList = null;
        if (StringUtils.isNotEmpty(channelIdArray)) {
            channelIdList = Arrays.stream(channelIdArray.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toList());
        }
        // 统计类型
        List<Integer> statTypes = getStatTypes(statTag);
        if (CollectionUtils.isEmpty(statTypes)) {
            log.info("暂无统计数据");
            return null;
        }

        List<StatPlatformVO> statDatas = statPlatformDAO.selectStat(channelIdList, statTypes, StatTimeType.YEAR_MONTH_DAY, startDate, endDate);

        middle.setTagType(statTag.getCode());
        middle.setTagName(statTag.getDesc());
        middle.setStartDate(startDate);
        middle.setEndDate(endDate);
        middle.setDateRangs(String.join(",", dataRangs.stream().map(e -> Convert.toStr(e)).collect(Collectors.toList())));
        List<OverallTrendMiddle.Data> datas = new ArrayList<>();
        for (Long date : dataRangs) {
            OverallTrendMiddle.Data data = new OverallTrendMiddle.Data();
            List<StatPlatformVO> filterByDate = statDatas.stream().filter(e -> date.equals(e.getStatTime())).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(filterByDate)) {
                if (statTypes.size() == 1) {
                    data.setValue1(filterByDate.stream().filter(e -> statTypes.get(0).equals(e.getStatType())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                } else if (statTypes.size() > 1) {
                    data.setValue1(filterByDate.stream().filter(e -> statTypes.get(0).equals(e.getStatType())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                    data.setValue2(filterByDate.stream().filter(e -> statTypes.get(1).equals(e.getStatType())).map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                }
            }
            data.setDate(date);
            data.setResult(caclResult(data.getValue1(), data.getValue2()));
            datas.add(data);
        }
        middle.setDatas(datas);
        middle.setStatType1(statTypes.get(0));
        middle.setStatTypeName1(StatPlatformTypeEnum.toEnum(middle.getStatType1()).getDesc());
        if (statTypes.size() > 1) {
            middle.setStatType2(statTypes.get(1));
            middle.setStatTypeName2(StatPlatformTypeEnum.toEnum(middle.getStatType2()).getDesc());
        }
        return middle;
    }

    @Override
    public Map<Long, List<OverallTrendBottom>> getOverallTrendBottom(Set<Long> channelIdSet, Long startDate, Long endDate) {

        Map<Long, List<OverallTrendBottom>> resultMap = new HashMap<>();

        // 日期
        List<Long> dataRangs = getDays(startDate, endDate);
        // 统计类型
        List<Integer> statTypes = Arrays.asList(new Integer[]{
                StatPlatformTypeEnum.DOWNLOAD_COUNT.getValue(),
                StatPlatformTypeEnum.INSTALL_COUNT.getValue(),
                StatPlatformTypeEnum.REGISTER_USERS.getValue(),
                StatPlatformTypeEnum.JOIN_ROOM_USERS.getValue(),
                StatPlatformTypeEnum.JOIN_RTC_ROOM_USERS.getValue(),
                StatPlatformTypeEnum.SINGLE_CHAT_TIMES_CONNECT.getValue(),
                StatPlatformTypeEnum.ACTIVE_USERS.getValue(),
                StatPlatformTypeEnum.LIVE_ANCHORS.getValue(),
                StatPlatformTypeEnum.RECHARGE_TIMES.getValue(),
                StatPlatformTypeEnum.RECHARGE_AMOUNT.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_AMOUNT.getValue(),
                StatPlatformTypeEnum.RECHARGE_USERS.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_USERS.getValue(),
        });

        List<StatPlatformVO> statDatas = statChannelDAO.selectStat(channelIdSet, statTypes, StatTimeType.YEAR_MONTH_DAY, startDate, endDate);

        for (Long date : dataRangs) {
            List<OverallTrendBottom> bottoms = new ArrayList<>();
            for (Integer statType : statTypes) {
                OverallTrendBottom bottomItem = new OverallTrendBottom();
                List<StatPlatformVO> filterByDate = statDatas.stream().filter(e -> statType.equals(e.getStatType()) && date.equals(e.getStatTime())).collect(Collectors.toList());
                bottomItem.setStatType(statType);
                bottomItem.setStatName(StatPlatformTypeEnum.toEnum(bottomItem.getStatType()).getDesc());
                bottomItem.setStatValue(filterByDate == null ? BigDecimal.ZERO : filterByDate.stream().map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                bottoms.add(bottomItem);
            }
            resultMap.put(date, bottoms);
        }
        return resultMap;
    }

    @Override
    public Map<Long, List<UserAccessTrend>> getUserAccessTrend(String channelIdArray, Long startDate, Long endDate) {

        Map<Long, List<UserAccessTrend>> resultMap = new HashMap<>();

        // 日期
        List<Long> dataRangs = getDays(startDate, endDate);
        // 渠道id
        List<Long> channelIdList = null;
        if (StringUtils.isNotEmpty(channelIdArray)) {
            channelIdList = Arrays.stream(channelIdArray.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toList());
        }
        // 统计类型
        List<Integer> statTypes = Arrays.asList(new Integer[]{
                StatPlatformTypeEnum.DOWNLOAD_COUNT.getValue(),
                StatPlatformTypeEnum.INSTALL_COUNT.getValue(),
                StatPlatformTypeEnum.REGISTER_USERS.getValue(),
                StatPlatformTypeEnum.JOIN_ROOM_USERS.getValue(),
                StatPlatformTypeEnum.JOIN_RTC_ROOM_USERS.getValue(),
                StatPlatformTypeEnum.SINGLE_CHAT_TIMES_CONNECT.getValue(),
                StatPlatformTypeEnum.ACTIVE_USERS.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_USERS.getValue(),
                StatPlatformTypeEnum.USER_SEX_UNKNOWN.getValue(),
                StatPlatformTypeEnum.USER_SEX_MALE.getValue(),
                StatPlatformTypeEnum.USER_SEX_FEMALE.getValue(),
        });

        List<StatPlatformVO> statDatas = statPlatformDAO.selectStat(channelIdList, statTypes, StatTimeType.YEAR_MONTH_DAY, startDate, endDate);

        for (Long date : dataRangs) {
            List<UserAccessTrend> trends = new ArrayList<>();
            for (Integer statType : statTypes) {
                UserAccessTrend trend = new UserAccessTrend();
                List<StatPlatformVO> filterByDate = statDatas.stream().filter(e -> statType.equals(e.getStatType()) && date.equals(e.getStatTime())).collect(Collectors.toList());
                trend.setStatType(statType);
                trend.setStatName(StatPlatformTypeEnum.toEnum(trend.getStatType()).getDesc());
                trend.setStatValue(filterByDate == null ? BigDecimal.ZERO : filterByDate.stream().map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                trends.add(trend);
            }
            resultMap.put(date, trends);
        }
        return resultMap;
    }

    @Override
    public Map<Long, List<UserPaymentTrend>> getUserPaymentTrend(String channelIdArray, Long startDate, Long endDate) {
        Map<Long, List<UserPaymentTrend>> resultMap = new HashMap<>();

        // 日期
        List<Long> dataRangs = getDays(startDate, endDate);
        // 渠道id
        List<Long> channelIdList = null;
        if (StringUtils.isNotEmpty(channelIdArray)) {
            channelIdList = Arrays.stream(channelIdArray.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toList());
        }
        // 统计类型
        List<Integer> statTypes = Arrays.asList(new Integer[]{
                StatPlatformTypeEnum.INSTALL_COUNT.getValue(),
                StatPlatformTypeEnum.REGISTER_USERS.getValue(),
                StatPlatformTypeEnum.ACTIVE_USERS.getValue(),
                StatPlatformTypeEnum.FIRST_RECHARGE_USERS.getValue(),
                StatPlatformTypeEnum.RECHARGE_TIMES.getValue(),
                StatPlatformTypeEnum.RECHARGE_AMOUNT.getValue()
        });

        List<StatPlatformVO> statDatas = statPlatformDAO.selectStat(channelIdList, statTypes, StatTimeType.YEAR_MONTH_DAY, startDate, endDate);

        for (Long date : dataRangs) {
            List<UserPaymentTrend> trends = new ArrayList<>();
            for (Integer statType : statTypes) {
                UserPaymentTrend trend = new UserPaymentTrend();
                List<StatPlatformVO> filterByDate = statDatas.stream().filter(e -> statType.equals(e.getStatType()) && date.equals(e.getStatTime())).collect(Collectors.toList());
                trend.setStatType(statType);
                trend.setStatName(StatPlatformTypeEnum.toEnum(trend.getStatType()).getDesc());
                trend.setStatValue(filterByDate == null ? BigDecimal.ZERO : filterByDate.stream().map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                trends.add(trend);
            }
            resultMap.put(date, trends);
        }

        return resultMap;
    }

    @Override
    public Map<Long, List<ChannelRegisterTrend>> getChannelRegisterTrend(String channelIdArray, Long startDate, Long endDate) {
        Map<Long, List<ChannelRegisterTrend>> resultMap = new HashMap<>();

        // 日期
        List<Long> dataRangs = getDays(startDate, endDate);
        // 渠道id
        List<Long> channelIdList = null;
        if (StringUtils.isNotEmpty(channelIdArray)) {
            channelIdList = Arrays.stream(channelIdArray.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toList());
        }
        // 统计类型
        List<Integer> statTypes = Arrays.asList(new Integer[]{
                StatPlatformTypeEnum.INSTALL_COUNT.getValue(),
                StatPlatformTypeEnum.REGISTER_USERS.getValue(),
                StatPlatformTypeEnum.REGISTER_BY_SMS.getValue(),
                StatPlatformTypeEnum.REGISTER_BY_ONE_KEY.getValue(),
                StatPlatformTypeEnum.REGISTER_BY_WX.getValue(),
                StatPlatformTypeEnum.REGISTER_BY_QQ.getValue(),
                StatPlatformTypeEnum.REGISTER_BY_WB.getValue()

        });

        List<StatPlatformVO> statDatas = statPlatformDAO.selectStat(channelIdList, statTypes, StatTimeType.YEAR_MONTH_DAY, startDate, endDate);

        for (Long date : dataRangs) {
            List<ChannelRegisterTrend> trends = new ArrayList<>();
            for (Integer statType : statTypes) {
                ChannelRegisterTrend trend = new ChannelRegisterTrend();
                List<StatPlatformVO> filterByDate = statDatas.stream().filter(e -> statType.equals(e.getStatType()) && date.equals(e.getStatTime())).collect(Collectors.toList());
                trend.setStatType(statType);
                trend.setStatName(StatPlatformTypeEnum.toEnum(trend.getStatType()).getDesc());
                trend.setStatValue(filterByDate == null ? BigDecimal.ZERO : filterByDate.stream().map(StatPlatformVO::getStatValue).reduce(BigDecimal.ZERO, BigDecimal::add));
                trends.add(trend);
            }
            resultMap.put(date, trends);
        }

        return resultMap;
    }

    /**
     * 获取统计类型
     *
     * @param statTag
     * @return
     */
    private List<Integer> getStatTypes(StatTagTypeEnum statTag) {
        // 统计类型
        List<Integer> statTypes = null;
        switch (statTag) {
            case ARPU:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.RECHARGE_AMOUNT.getValue(),
                        StatPlatformTypeEnum.REGISTER_USERS.getValue()
                });
                break;
            case NEW_ARPU:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.FIRST_RECHARGE_AMOUNT.getValue(),
                        StatPlatformTypeEnum.REGISTER_USERS.getValue()
                });
                break;
            case DOWNLOAD:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.DOWNLOAD_COUNT.getValue()
                });
                break;
            case NEW_USER:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.REGISTER_USERS.getValue()
                });
                break;
            case PAY_RATE:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.RECHARGE_USERS.getValue(),
                        StatPlatformTypeEnum.REGISTER_USERS.getValue()
                });
                break;
            case NEW_PAY_RATE:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.FIRST_RECHARGE_AMOUNT.getValue(),
                        StatPlatformTypeEnum.FIRST_RECHARGE_AMOUNT.getValue()
                });
                break;
            case CALL_RATE:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.SINGLE_CHAT_TIMES_CONNECT.getValue(),
                        StatPlatformTypeEnum.SINGLE_CHAT_TIMES_REQUEST.getValue()
                });
                break;
            case CALL_TIMES:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.SINGLE_CHAT_TIMES_REQUEST.getValue()
                });
                break;
            case ACTIVE_USER:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.ACTIVE_USERS.getValue()
                });
                break;
            case NEW_RECHARGE:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.FIRST_RECHARGE_USERS.getValue()
                });
                break;
            case VIP_RECHARGE:
                break;
            case REGISTER_USER:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.REGISTER_USERS.getValue()
                });
                break;
            case VIP_COUNT:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.STAT_VIP_NEW.getValue()
                });
                break;
            case RECHARGE_USERS:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.RECHARGE_USERS.getValue()
                });
                break;
            case RECHARGE_AMOUNT:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.RECHARGE_AMOUNT.getValue()
                });
                break;
            case NEW_RECHARGE_USERS:
                statTypes = Arrays.asList(new Integer[]{
                        StatPlatformTypeEnum.FIRST_RECHARGE_USERS.getValue()
                });
                break;
            case AVERAGE_NUMBER_TARTS:
                break;
        }
        return statTypes;
    }

    public static List<Long> getDays(Long startTime, Long endTime) {
        // 返回的日期集合
        List<Long> days = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        try {
            Date start = sdf.parse(Convert.toStr(startTime));
            Date end = sdf.parse(Convert.toStr(endTime));

            Calendar tempStart = Calendar.getInstance();
            tempStart.setTime(start);

            Calendar tempEnd = Calendar.getInstance();
            tempEnd.setTime(end);
            tempEnd.add(Calendar.DATE, +1);// 日期加1(包含结束)
            while (tempStart.before(tempEnd)) {
                days.add(Convert.toLong(sdf.format(tempStart.getTime())));
                tempStart.add(Calendar.DAY_OF_YEAR, 1);
            }

        } catch (ParseException e) {
            e.printStackTrace();
        }
        return days;
    }

    public static void main(String[] args) throws ParseException {
        Set<Integer> typeSet = Arrays.stream(StatPlatformTypeEnum.values()).map(StatPlatformTypeEnum::getValue).collect(Collectors.toSet());
        String statTypes = StringUtils.join(typeSet, ",");
        System.out.println(statTypes);
        List<Long> days = getDays(20200301L, 20200330L);
        System.out.println(days);
    }

}
