package com.starbuds.server.api.channel.service.admin.impl;

import com.starbuds.server.api.channel.common.pojo.admin.UserVO;
import com.starbuds.server.api.channel.common.service.admin.ChUserService;
import com.starbuds.server.api.channel.service.admin.dao.UserDAO;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.finance.ScoreTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.user.UserAlipayVO;
import com.starbuds.server.common.service.data.UserSearchService;
import com.starbuds.server.common.service.user.ChannelService;
import com.starbuds.server.common.service.user.SocialRelationService;
import com.starbuds.server.common.service.user.ThirdAuthService;
import com.starbuds.server.common.service.user.UserService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/4/22 14:39
 * @Description
 */
@Service
public class ChUserServiceImpl implements ChUserService {

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Reference(version = "1.0.0")
    SocialRelationService socialRelationService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    UserDAO userDAO;

    @Reference(version = "1.0.0")
    ThirdAuthService thirdAuthService;

    @Override
    public PageList<UserVO> getUserList(Long userNo, String userName, Long niceNo, String userPhone, String idCardNo,
                                        Integer isAnchor, Integer userStatus, Long channelId, int pageNum, int pageSize, int sortType, int isAsc, Long startTime, Long endTime) {
        PageList<UserVO> page = new PageList<>(pageNum, pageSize);
        PageList<Long> pageList = userSearchService.getUserPageList(userNo, niceNo, userName, userStatus, userPhone, idCardNo,
                isAnchor, channelId, pageNum, pageSize, sortType, isAsc == BoolIntVal.True, startTime, endTime);
        List<Long> userIds = pageList.getList();
        page.setTotal(pageList.getTotal());

        if (CollectionUtils.isEmpty(userIds)) {
            page.setList(new ArrayList<>());
            return page;
        }

        List<UserVO> userList = new ArrayList<>();
        for (Long userId : userIds) {
            UserVO vo = userDAO.selectDetail(userId);
            if (vo == null) {
                vo = new UserVO();
                vo.setUserName("该用户已经找不到了");
            }
            vo.setChannelName(channelService.getChannelName(vo.getChannelId()));
            vo.setFansNum(socialRelationService.getFansCount(userId));
            vo.setFollowNum(socialRelationService.getFollowCount(userId));
            vo.setWealthValue(userService.getScoreValue(userId, ScoreTypeEnum.WEALTH.getValue()));
            vo.setCharmValue(userService.getScoreValue(userId, ScoreTypeEnum.CHARM.getValue()));

            Long inviterUserId = userService.getInviterUserId(vo.getUserId());

            vo.setInviterUserId(inviterUserId);
            vo.setInviterUserName(userDAO.selectUserName(inviterUserId));

            userList.add(vo);
        }
        page.setList(userList);

        return page;

    }

    @Override
    public UserVO getUserDetail(Long userId) {
        UserVO vo = userDAO.selectDetail(userId);
        if (vo == null) {
            vo = new UserVO();
            vo.setUserName("该用户已经找不到了");
        }
        vo.setChannelName(channelService.getChannelName(vo.getChannelId()));
        vo.setFansNum(socialRelationService.getFansCount(userId));
        vo.setFollowNum(socialRelationService.getFollowCount(userId));
        vo.setWealthValue(userService.getScoreValue(userId, ScoreTypeEnum.WEALTH.getValue()));
        vo.setCharmValue(userService.getScoreValue(userId, ScoreTypeEnum.CHARM.getValue()));

        Long inviterUserId = userService.getInviterUserId(vo.getUserId());
        vo.setInviterUserId(inviterUserId);
        vo.setInviterUserName(userDAO.selectUserName(inviterUserId));

        UserAlipayVO userAlipayVO = thirdAuthService.getAlipay(userId);
        if (userAlipayVO != null) {
            vo.setAlipayAccount(hideAlipayAccount(userAlipayVO.getAlipayAccount()));
        }

        return vo;
    }

    private static String hideAlipayAccount(String str) {
        if (str.indexOf('@') > -1) {
            return str.replaceAll("(\\w?)(\\w+)(\\w)(@\\w+\\.[a-z]+(\\.[a-z]+)?)", "$1****$3$4");
        } else {
            return str.replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
        }
    }
}
