package com.starbuds.server.api.channel.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/16 14:03
 * @Description 管理员管理
 */
public interface AdminFacade {


    /**
     * 账号密码登录
     *
     * @param account
     * @param password
     * @return
     */
    Result login(String account, String password);

    /**
     * 个人信息
     *
     * @return
     */
    Result getInfo();


    /**
     * 重置密码
     *
     * @param adminId 被重置人的id
     * @param newPwd  新密码
     * @return result对象
     */
    Result resetPwd(Long adminId, String newPwd);


    /**
     * 修改密码
     *
     * @param oldPwd 旧密码
     * @param newPwd 新密码
     * @return result对象
     */
    Result changePwd(String oldPwd, String newPwd);

    /**
     * 创建管理员
     *
     * @param adminName      管理员名称
     * @param adminPhone     手机号
     * @param adminAvatar    头像
     * @param adminEmail     email
     * @param adminPass      密码
     * @param status         状态
     * @param authChannelIds 授权渠道id
     * @return result对象
     */
    Result createAdmin(String adminName, String adminPhone, String adminAvatar, String adminEmail, String adminPass, Integer status, String authChannelIds);


    /**
     * 获取管理员列表
     *
     * @param adminName  管理员姓名
     * @param adminPhone 手机号
     * @param status     状态
     * @param pageIndex  当前页码
     * @param pageSize   分页大小
     * @return result对象
     */
    Result getAdminPageList(String adminName, String adminPhone, Integer status, int pageIndex, int pageSize);


    /**
     * 编辑管理员
     *
     * @param adminId     管理员id
     * @param adminName   管理员名称
     * @param adminAvatar 头像
     * @param adminEmail  email
     * @param status      状态
     * @return result对象
     */
    Result editAdmin(Long adminId, String adminName, String adminAvatar, String adminEmail, Integer status, String authChannelIds);


    /**
     * 禁用管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    Result disableAdmin(Long adminId);

    /**
     * 启用管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    Result enableAdmin(Long adminId);

    /**
     * 删除管理员
     *
     * @param adminId 管理员id
     * @return result对象
     */
    Result deleteAdmin(Long adminId);


}
