package com.starbuds.server.api.channel.v1.facade;

import com.starbuds.server.api.channel.common.pojo.admin.ChannelAppReq;
import com.starbuds.server.api.channel.common.pojo.admin.ChannelRatioReq;
import com.starbuds.server.common.pojo.api.Result;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/16 14:03
 * @Description 渠道管理
 */
public interface ChannelFacade {

    /**
     * 获取渠道信息
     *
     * @param channelId
     * @return
     */
    Result getInfo(Long channelId);

    /**
     * 编辑渠道 App
     *
     * @param channelReq
     * @return
     */
    Result editChannelApp(ChannelAppReq channelReq);

    /**
     * 获得渠道 App 信息
     *
     * @return
     */
    Result getChannelApp(Long channelId);

    /**
     * 生成渠道 APP 秘钥
     *
     * @return
     */
    Result createSecretKey(Long channelId);

    /**
     * 变更渠道APP秘钥
     *
     * @param secretId
     * @return
     */
    Result changeSecretKey(Long secretId, Long channelId);

    /**
     * 删除渠道APP秘钥
     *
     * @param secretId
     * @return
     */
    Result deleteSecretKey(Long secretId, Long channelId);

    /**
     * 启用/禁用渠道APP秘钥
     *
     * @param secretId
     * @param isActivate 1:启用; 0:禁用
     * @return
     */
    Result activateSecretKey(Long secretId, Integer isActivate, Long channelId);

    /**
     * 查询（自己渠道下）所有未删除的秘钥
     *
     * @return
     */
    Result getSecretKeyList(Long channelId);

    /**
     * 获取渠道分组下的渠道集合
     *
     * @return
     */
    Result getChannelListByGroupId();

    /**
     * 设置渠道分成比例
     *
     * @param ratioReq
     * @return
     */
    Result setChannelRatio(ChannelRatioReq ratioReq);

    /**
     * 获取渠道列表
     *
     * @param channelName
     * @param status
     * @param pageIndex
     * @param pageSize
     * @return
     */
    Result getChannelPageList(String channelName, Integer status, int pageIndex, int pageSize);

}
