package com.starbuds.server.api.channel.v1.impl;

import com.starbuds.server.api.channel.common.pojo.constant.NotifyApiEnum;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.ChannelNotifyConfigDO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyConfigService;
import com.starbuds.server.api.channel.common.pojo.form.ChannelNotifyConfigForm;
import com.starbuds.server.api.channel.common.pojo.qo.ChNotifyConfigPageListQO;
import com.starbuds.server.api.channel.v1.facade.ChannelNotifyConfigFacade;

import javax.annotation.Resource;
import javax.ws.rs.*;

import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import lombok.extern.slf4j.Slf4j;

import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.constant.common.GlobalApiError;

import org.apache.dubbo.rpc.protocol.rest.support.ContentType;

import static com.starbuds.server.common.utils.CheckFormUtil.*;

/**
 * 渠道异步通知配置控制器
 */
@Path("/v1/channel/notify/cfg/")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ChannelNotifyConfigFacadeImpl extends BaseFacade implements ChannelNotifyConfigFacade {

    @Resource
    private ChChannelNotifyConfigService chChannelNotifyConfigService;

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    @GET
    @Path("exist")
    @Override
    public Result exist(@QueryParam("id") Long id) {
        if (id == null) {
            return new Result(GlobalApiError.BadRequest, "id不能为空");
        }
        return new Result(chChannelNotifyConfigService.exist(id));
    }

    /**
     * 分页查询
     */
    @GET
    @Path("getPageList")
    @Override
    public Result getPageList(@BeanParam ChNotifyConfigPageListQO qo) {
        return new Result(chChannelNotifyConfigService.getVOPageList(qo));
    }

    // 手动编码开始

    /**
     * 查询详细信息
     *
     * @param apiNo 接口编号
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo(@QueryParam("apiNo") Integer apiNo) {
        if (apiNo == null) {
            return new Result(GlobalApiError.BadRequest, "apiNo必传");
        }
        NotifyApiEnum notifyApiEnum = NotifyApiEnum.toEnum(apiNo);
        if (notifyApiEnum == null) {
            return new Result(GlobalApiError.BadRequest, "无效的apiNo");
        }
        return new Result(chChannelNotifyConfigService.getNotifyCfgVO(getChannelId(), notifyApiEnum));
    }

    /**
     * 更新数据
     */
    @POST
    @Path("update")
    @Override
    public Result update(@BeanParam ChannelNotifyConfigForm form) {
        String check = checkUpdate(form);
        if (check != null) {
            return new Result(GlobalApiError.BadRequest, check);
        }
        Integer apiNo = form.getApiNo();
        NotifyApiEnum notifyApiEnum = NotifyApiEnum.toEnum(apiNo);
        if (notifyApiEnum == null) {
            return new Result(GlobalApiError.BadRequest, "无效的apiNo");
        }
        Long channelId = getChannelId();
        if (channelId == null) {
            return new Result(GlobalApiError.BadRequest, "无法获得channelId，请重新登录");
        }
        ChannelNotifyConfigDO po = new ChannelNotifyConfigDO();
        po.setChannelId(channelId);
        BeanUtils.copyProperties(form, po);
        String host = form.getHostUrl();
        if (!host.endsWith("/")) {
            host += "/";
        }
        po.setHostUrl(host);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq("channelId", channelId);
        qw.eq("apiNo", apiNo);
        uw.setQw(qw);
        boolean flag = chChannelNotifyConfigService.update(uw);

        if (!flag) {
            return new Result(GlobalApiError.OperationFailed, "更新失败");
        }
        return new Result("更新成功");
    }
}
