package com.starbuds.server.api.channel.v1.impl;

import com.starbuds.server.api.channel.common.pojo.qo.ChNotifyHisPageListQO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelNotifyHisService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelRedEnvelopeHisService;
import com.starbuds.server.api.channel.v1.facade.ChannelNotifyHisFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.ChannelNotifyHisDO;
import com.starbuds.server.common.pojo.db.ChannelRedEnvelopeHisDO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;

/**
 * 渠道异步通知记录控制器
 */
@Path("/v1/channel/notify/his/")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ChannelNotifyHisFacadeImpl extends BaseFacade implements ChannelNotifyHisFacade {

    @Resource
    private ChChannelNotifyHisService chChannelNotifyHisService;

    @Resource
    private ChChannelRedEnvelopeHisService chChannelRedEnvelopeHisService;

    /**
     * 查询详细信息
     *
     * @param id 主键
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo(@QueryParam("id") Long id) {
        if (id == null) {
            return new Result(GlobalApiError.BadRequest, "id不能为空");
        }
        return new Result(chChannelNotifyHisService.getVOById(id));
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    @GET
    @Path("exist")
    @Override
    public Result exist(@QueryParam("id") Long id) {
        if (id == null) {
            return new Result(GlobalApiError.BadRequest, "id不能为空");
        }
        return new Result(chChannelNotifyHisService.exist(id));
    }

    /**
     * 分页查询
     */
    @GET
    @Path("getPageList")
    @Override
    public Result getPageList(@BeanParam ChNotifyHisPageListQO qo) {
        qo.setChannelId(getChannelId());
        return new Result(chChannelNotifyHisService.getVOPageList(qo));
    }

    // 手动编码开始

    /**
     * 重发通知消息
     *
     * @param hisId 渠道异步通知记录id
     * @return
     */
    @Path("resendNotification")
    @POST
    @Override
    public Result resendNotification(@FormParam("hisId") Long hisId) {
        if (hisId == null) {
            return new Result(GlobalApiError.BadRequest, "hisId不能为空");
        }
        ChannelNotifyHisDO notifyHis = chChannelNotifyHisService.getById(hisId);
        if (notifyHis == null) {
            return new Result(GlobalApiError.BadRequest, "无效的hisId");
        }
        if (getChannelId() - notifyHis.getChannelId() != 0) {
            return new Result(GlobalApiError.BadRequest, "只能重发自己渠道的通知");
        }
        ChannelRedEnvelopeHisDO redEnvHis = chChannelRedEnvelopeHisService.getById(notifyHis.getTraceId());
        chChannelNotifyHisService.resendNotification(hisId, notifyHis, redEnvHis);
        return new Result();
    }
}
