package com.starbuds.server.api.channel.v1.impl;

import com.starbuds.server.api.channel.common.pojo.db.ChannelSecretRsaDO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelSecretRsaService;
import com.starbuds.server.api.channel.common.pojo.form.ChannelSecretRsaForm;
import com.starbuds.server.api.channel.common.pojo.qo.ChSecretRsaPageListQO;
import com.starbuds.server.api.channel.v1.facade.ChannelSecretRsaFacade;

import javax.annotation.Resource;
import javax.ws.rs.*;

import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import lombok.extern.slf4j.Slf4j;

import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.constant.common.GlobalApiError;

import org.apache.dubbo.rpc.protocol.rest.support.ContentType;

import static com.starbuds.server.common.utils.CheckFormUtil.*;

/**
 * 渠道RSA2信息控制器
 */
@Path("/v1/channel/sec/rsa")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ChannelSecretRsaFacadeImpl extends BaseFacade implements ChannelSecretRsaFacade {

    @Resource
    private ChChannelSecretRsaService chChannelSecretRsaService;

    /**
     * 更新数据
     */
    @POST
    @Path("update")
    @Override
    public Result update(@BeanParam ChannelSecretRsaForm form) {
        String check = checkUpdate(form);
        if (check != null) {
            return new Result(GlobalApiError.BadRequest, check);
        }
        Long channelId = getChannelId(form.getChannelId());
        if (channelId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的渠道id");
        }
        ChannelSecretRsaDO rsa = chChannelSecretRsaService.getRsa(channelId);
        if (rsa == null) {
            return new Result(GlobalApiError.BadRequest, "RSA信息不存在");
        }

        ChannelSecretRsaDO po = new ChannelSecretRsaDO();
        po.setChSecretRsaId(rsa.getChSecretRsaId());
        po.setPubKey(form.getPubKey());
        po.setAdminId(getAdminId());
        BeanUtils.copyProperties(form, po);
        boolean flag = chChannelSecretRsaService.updateById(po);

        if (!flag) {
            return new Result(GlobalApiError.OperationFailed, "更新失败");
        }
        return new Result("更新成功");
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    @GET
    @Path("exist")
    @Override
    public Result exist(@QueryParam("id") Long id) {
        if (id == null) {
            return new Result(GlobalApiError.BadRequest, "id不能为空");
        }
        return new Result(chChannelSecretRsaService.exist(id));
    }

    /**
     * 分页查询
     */
    @GET
    @Path("getPageList")
    @Override
    public Result getPageList(@BeanParam ChSecretRsaPageListQO qo) {
        return new Result(chChannelSecretRsaService.getVOPageList(qo));
    }

    // 手动编码开始

    /**
     * 查询详细信息
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo(@QueryParam("channelId") Long channelId) {
        channelId = getChannelId(channelId);
        if (channelId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的渠道id");
        }
        return new Result(chChannelSecretRsaService.getRsaVO(channelId));
    }
}
