package com.starbuds.server.api.channel.v1.impl;

import com.starbuds.server.api.channel.common.pojo.qo.ChWalletPageListQO;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelRechargeVO;
import com.starbuds.server.api.channel.common.pojo.vo.ChannelWalletVO;
import com.starbuds.server.api.channel.common.service.admin.ChChannelRechargeHisService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelWalletService;
import com.starbuds.server.api.channel.v1.facade.ChannelWalletFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.finance.ChannelPayOrder;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;

/**
 * 渠道钱包控制器
 */
@Path("/v1/channel/wallet/")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ChannelWalletFacadeImpl extends BaseFacade implements ChannelWalletFacade {

    @Resource
    private ChChannelWalletService chChannelWalletService;

    @Resource
    private ChChannelRechargeHisService chChannelRechargeHisService;

    @Reference(version = "1.0.0")
    private UserService userService;

    /**
     * 分页查询
     */
    @GET
    @Path("getPageList")
    @Override
    public Result getPageList(@BeanParam ChWalletPageListQO qo) {
        return new Result(chChannelWalletService.getVOPageList(qo));
    }

    // 手动编码开始

    /**
     * 查询详细信息(查询或初始化)
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo() {
        ChannelWalletVO vo = chChannelWalletService.getWalletVO(getChannelId());
        if (vo == null) {
            vo = new ChannelWalletVO();
            vo.setBalance(0L);
        }
        return new Result(vo);
    }

    /**
     * 充值（星钻到渠道钱包）
     *
     * @param amount 支付金额（RMB）
     * @return
     */
    @POST
    @Path("recharge")
    @Override
    public Result recharge(@FormParam("amount") Long amount) {
        if (amount == null || amount < 1) {
            return new Result(GlobalApiError.BadRequest, "充值金额必须大于0");
        }
        ChannelRechargeVO vo = chChannelRechargeHisService.recharge(getChannelId(), amount);
        if (vo == null) {
            return new Result(GlobalApiError.OperationFailed, "调用失败，请重试");
        }
        return new Result(vo);
    }

    /**
     * 查询渠道充值订单支付状态
     *
     * @param orderId
     * @return PayStatusEnum 11:待支付; 21:支付失败; 22:支付取消; 99:支付成功
     */
    @GET
    @Path("getPayStatus")
    @Override
    public Result getPayStatus(@QueryParam("orderId") Long orderId) {
        if (orderId == null || orderId < 1) {
            return new Result(GlobalApiError.BadRequest, "订单号必须大于0");
        }
        Integer status = chChannelRechargeHisService.getPayStatus(getChannelId(), orderId);
        if (status == null) {
            return new Result(GlobalApiError.OperationFailed, "订单状态不存在");
        }
        return new Result(status);
    }

    /**
     * 发星钻红包
     *
     * @param userNo 星芽号
     * @param amount
     * @return
     */
    @POST
    @Path("sendDiamond")
    @Override
    public Result sendDiamond(@FormParam("userNo") Long userNo, @FormParam("amount") Long amount) {
        if (amount < 1) {
            return new Result(GlobalApiError.BadRequest, "红包金额必须大于0");
        }
        Long userId = userService.getUserIdByNo(userNo);
        if (userId == null) {
            return new Result(GlobalApiError.BadRequest, "无效的星芽号");
        }

        Long balance = chChannelWalletService.sendDiamond(getChannelId(), userId, amount, getAdminId(), getIpAddress());
        if (balance == null) {
            return new Result(GlobalApiError.OperationFailed, "发放红包失败");
        } else if (balance < 0) {
            return new Result(GlobalApiError.OperationFailed, "余额不足");
        }
        return new Result("发放红包成功");
    }
}
