package com.starbuds.server.api.channel.v1.impl;

import com.starbuds.server.api.channel.common.service.admin.ChAdminSecurityService;
import com.starbuds.server.api.channel.v1.facade.DataFacade;
import com.starbuds.server.api.manage.common.pojo.admin.TokenParam;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.data.FileService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;

/**
 * 数据接口实现
 *
 * @author aoe
 * @date 2020/6/5
 */
@Path("/v1/data")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class DataFacadeImpl extends BaseFacade implements DataFacade {

    @Reference(version = "1.0.0")
    private FileService fileService;

    @Resource
    private ChAdminSecurityService chAdminSecurityService;

    /**
     * 获取第三方app icon上传配置
     *
     * @return
     */
    @GET
    @Path("getThirdPartyAppIconUploadConfig")
    @Override
    public Result getThirdPartyAppIconUploadConfig(@QueryParam("channelId") Long channelId) {
        TokenParam token = getToken();
        if (token == null) {
            return new Result(GlobalApiError.BadRequest, "请登录");
        }
        if (!chAdminSecurityService.hasPermissions(token.getAdminId(), getChannelId(channelId))) {
            log.warn("您没有这个渠道的权限 adminId={} channelId={}", token.getAdminId(), channelId);
            return new Result(GlobalApiError.BadRequest, "您没有这个渠道的权限");
        }

        String key = fileService.getThirdPartyAppIconUploadKey();
        String upToken = fileService.getMediaUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }
}
