package com.starbuds.server.api.channel.v1.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.channel.common.pojo.admin.*;
import com.starbuds.server.api.channel.common.service.admin.ChAdminService;
import com.starbuds.server.api.channel.common.service.admin.ChChannelService;
import com.starbuds.server.api.channel.common.service.admin.ChStatPlatformService;
import com.starbuds.server.api.channel.v1.facade.StatFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 统计管理
 *
 * @author jiangxiaowei
 * @date 2020/2/18 16:02
 */
@Path("/v1/stat")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class StatFacadeImpl extends BaseFacade implements StatFacade {

    @Autowired
    ChStatPlatformService chStatPlatformService;

    @Resource
    ChAdminService chAdminService;

    @Resource
    ChChannelService chChannelService;

    /**
     * 获取统计列表
     *
     * @param statType
     * @param statTimeType
     * @param pageIndex
     * @param pageSize
     * @param channelIdArray
     * @return
     */
    @GET
    @Path("getStatPlatformPageList")
    @Override
    public Result getStatPlatformPageList(@QueryParam("statType") Integer statType,
                                          @QueryParam("statTimeType") Integer statTimeType,
                                          @QueryParam("pageIndex") int pageIndex,
                                          @QueryParam("pageSize") int pageSize,
                                          @QueryParam("channelIdArray") String channelIdArray) {

        Set<Long> channelIdSet = new HashSet<>();
        Long channelId = this.getToken().getChannelId();
        Long adminId = this.getToken().getAdminId();

        Set<Long> channelIds = new HashSet<>();
        // 母账号
        if (chAdminService.isParent(adminId)) {
            Long channelGroupId = chAdminService.getChannelGroupId(adminId);
            List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
            channelIds.addAll(dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet()));
        } else {
            // 子账号
            String authChannelIds = chAdminService.getAuthChannelIds(adminId);
            if (StringUtils.isNotEmpty(authChannelIds)) {
                Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                channelIds.addAll(authChannelIdSet);
            }
        }

        if (!CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            String[] split = channelIdArray.split(",");
            for (String item : split) {
                if (StringUtils.isNotEmpty(item) && (!channelIds.contains(Convert.toLong(item)) && !Convert.toLong(item).equals(channelId))) {
                    return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
                }
                channelIdSet.add(Convert.toLong(item));
            }
        } else if (CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            if (!channelIdArray.equals(Convert.toStr(channelId))) {
                return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
            }
            channelIdSet.add(Convert.toLong(channelId));
        } else if (StringUtils.isEmpty(channelIdArray)) {
            channelIdSet.add(channelId);
            channelIdSet.addAll(channelIds);
        }
        return new Result(chStatPlatformService.getStatPlatformPageList(channelIdSet, statType, statTimeType, pageIndex, pageSize));
    }

    /**
     * 获取平台总统计
     *
     * @return
     */
    @GET
    @Path("getCurrentAndYesterdayStat")
    @Override
    public Result getCurrentAndYesterdayStat() {
        List<CurrentAndYesterdayStatEntity> statList = chStatPlatformService.getCurrentAndYesterdayStat(this.getToken().getChannelId());
        MutableResult mResult = new MutableResult();
        mResult.put("list", statList);
        return mResult.build();
    }

    /**
     * 获取统计详情
     *
     * @param timeType
     * @param startTime
     * @param endTime
     * @param statType
     * @return
     */
    @GET
    @Path("getStatDetail")
    @Override
    public Result getStatDetail(@QueryParam("timeType") Integer timeType,
                                @QueryParam("startTime") Long startTime,
                                @QueryParam("endTime") Long endTime,
                                @QueryParam("statType") Integer statType) {
        return new Result(chStatPlatformService.getStatDetail(this.getToken().getChannelId(), timeType, startTime, endTime, statType));
    }

    /**
     * 时段概况 顶部
     *
     * @param startDate
     * @param endDate
     * @param channelIdArray
     * @return
     */
    @GET
    @Path("getPeriodOverviewTop")
    @Override
    public Result getPeriodOverviewTop(@QueryParam("startDate") Long startDate,
                                       @QueryParam("endDate") Long endDate,
                                       @QueryParam("channelIdArray") String channelIdArray) {
        if (startDate == null || endDate == null) {
            return new Result(GlobalApiError.BadRequest);
        }

        Set<Long> channelIdSet = new HashSet<>();
        Long channelId = this.getToken().getChannelId();
        Long adminId = this.getToken().getAdminId();

        Set<Long> channelIds = new HashSet<>();
        // 母账号
        if (chAdminService.isParent(adminId)) {
            Long channelGroupId = chAdminService.getChannelGroupId(adminId);
            List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
            channelIds.addAll(dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet()));
        } else {
            // 子账号
            String authChannelIds = chAdminService.getAuthChannelIds(adminId);
            if (StringUtils.isNotEmpty(authChannelIds)) {
                Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                channelIds.addAll(authChannelIdSet);
            }
        }

        if (!CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            String[] split = channelIdArray.split(",");
            for (String item : split) {
                if (StringUtils.isNotEmpty(item) && (!channelIds.contains(Convert.toLong(item)) && !Convert.toLong(item).equals(channelId))) {
                    return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
                }
                channelIdSet.add(Convert.toLong(item));
            }
        } else if (CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            if (!channelIdArray.equals(Convert.toStr(channelId))) {
                return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
            }
            channelIdSet.add(Convert.toLong(channelId));
        } else if (StringUtils.isEmpty(channelIdArray)) {
            channelIdSet.add(channelId);
            channelIdSet.addAll(channelIds);
        }

        String joinChannelId = CollectionUtil.join(channelIdSet, ",");

        return new Result(chStatPlatformService.getPeriodOverviewTop(joinChannelId, startDate, endDate));
    }

    /**
     * 时段概况 中间
     *
     * @param startDate
     * @param endDate
     * @param tagType
     * @param channelIdArray
     * @return
     */
    @GET
    @Path("getPeriodOverviewMiddle")
    @Override
    public Result getPeriodOverviewMiddle(@QueryParam("startDate") Long startDate,
                                          @QueryParam("endDate") Long endDate,
                                          @QueryParam("tagType") Integer tagType,
                                          @QueryParam("channelIdArray") String channelIdArray) {
        if (startDate == null || endDate == null || tagType == null) {
            return new Result(GlobalApiError.BadRequest);
        }

        Set<Long> channelIdSet = new HashSet<>();
        Long channelId = this.getToken().getChannelId();
        Long adminId = this.getToken().getAdminId();

        Set<Long> channelIds = new HashSet<>();
        // 母账号
        if (chAdminService.isParent(adminId)) {
            Long channelGroupId = chAdminService.getChannelGroupId(adminId);
            List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
            channelIds.addAll(dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet()));
        } else {
            // 子账号
            String authChannelIds = chAdminService.getAuthChannelIds(adminId);
            if (StringUtils.isNotEmpty(authChannelIds)) {
                Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                channelIds.addAll(authChannelIdSet);
            }
        }

        if (!CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            String[] split = channelIdArray.split(",");
            for (String item : split) {
                if (StringUtils.isNotEmpty(item) && (!channelIds.contains(Convert.toLong(item)) && !Convert.toLong(item).equals(channelId))) {
                    return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
                }
                channelIdSet.add(Convert.toLong(item));
            }
        } else if (CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            if (!channelIdArray.equals(Convert.toStr(channelId))) {
                return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
            }
            channelIdSet.add(Convert.toLong(channelId));
        } else if (StringUtils.isEmpty(channelIdArray)) {
            channelIdSet.add(channelId);
            channelIdSet.addAll(channelIds);
        }

        String joinChannelId = CollectionUtil.join(channelIdSet, ",");

        StatPeriodOverViewMiddle overviewMiddle = chStatPlatformService.getPeriodOverviewMiddle(joinChannelId, startDate, endDate, tagType);
        if (overviewMiddle == null) {
            return new Result(GlobalApiError.OperationFailed, "暂无统计数据");
        }
        return new Result(overviewMiddle);
    }

    /**
     * 时段概况 底部
     *
     * @param startDate
     * @param endDate
     * @param channelIdArray
     * @return
     */
    @GET
    @Path("getPeriodOverviewBottom")
    @Override
    public Result getPeriodOverviewBottom(@QueryParam("startDate") Long startDate,
                                          @QueryParam("endDate") Long endDate,
                                          @QueryParam("channelIdArray") String channelIdArray) {
        if (startDate == null || endDate == null) {
            return new Result(GlobalApiError.BadRequest);
        }

        Set<Long> channelIdSet = new HashSet<>();
        Long channelId = this.getToken().getChannelId();
        Long adminId = this.getToken().getAdminId();

        Set<Long> channelIds = new HashSet<>();
        // 母账号
        if (chAdminService.isParent(adminId)) {
            Long channelGroupId = chAdminService.getChannelGroupId(adminId);
            List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
            channelIds.addAll(dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet()));
        } else {
            // 子账号
            String authChannelIds = chAdminService.getAuthChannelIds(adminId);
            if (StringUtils.isNotEmpty(authChannelIds)) {
                Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                channelIds.addAll(authChannelIdSet);
            }
        }

        if (!CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            String[] split = channelIdArray.split(",");
            for (String item : split) {
                if (StringUtils.isNotEmpty(item) && (!channelIds.contains(Convert.toLong(item)) && !Convert.toLong(item).equals(channelId))) {
                    return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
                }
                channelIdSet.add(Convert.toLong(item));
            }
        } else if (CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            if (!channelIdArray.equals(Convert.toStr(channelId))) {
                return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
            }
            channelIdSet.add(Convert.toLong(channelId));
        } else if (StringUtils.isEmpty(channelIdArray)) {
            channelIdSet.add(channelId);
            channelIdSet.addAll(channelIds);
        }

        String joinChannelId = CollectionUtil.join(channelIdSet, ",");

        Map<String, List<StatPeriodOverViewBottom>> overviewBottom = chStatPlatformService.getPeriodOverviewBottom(joinChannelId, startDate, endDate);
        if (overviewBottom == null) {
            return new Result(GlobalApiError.OperationFailed, "暂无统计数据");
        }
        return new Result(overviewBottom);
    }

    /**
     * 整体趋势 顶部
     *
     * @param startDate
     * @param endDate
     * @param channelIdArray
     * @return
     */
    @GET
    @Path("getOverallTrendTop")
    @Override
    public Result getOverallTrendTop(@QueryParam("startDate") Long startDate,
                                     @QueryParam("endDate") Long endDate,
                                     @QueryParam("channelIdArray") String channelIdArray) {
        if (startDate == null || endDate == null) {
            return new Result(GlobalApiError.BadRequest);
        }

        Set<Long> channelIdSet = new HashSet<>();
        Long channelId = this.getToken().getChannelId();
        Long adminId = this.getToken().getAdminId();

        Set<Long> channelIds = new HashSet<>();
        // 母账号
        if (chAdminService.isParent(adminId)) {
            Long channelGroupId = chAdminService.getChannelGroupId(adminId);
            List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
            channelIds.addAll(dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet()));
        } else {
            // 子账号
            String authChannelIds = chAdminService.getAuthChannelIds(adminId);
            if (StringUtils.isNotEmpty(authChannelIds)) {
                Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                channelIds.addAll(authChannelIdSet);
            }
        }

        if (!CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            String[] split = channelIdArray.split(",");
            for (String item : split) {
                if (StringUtils.isNotEmpty(item) && (!channelIds.contains(Convert.toLong(item)) && !Convert.toLong(item).equals(channelId))) {
                    return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
                }
                channelIdSet.add(Convert.toLong(item));
            }
        } else if (CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            if (!channelIdArray.equals(Convert.toStr(channelId))) {
                return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
            }
            channelIdSet.add(Convert.toLong(channelId));
        } else if (StringUtils.isEmpty(channelIdArray)) {
            channelIdSet.add(channelId);
            channelIdSet.addAll(channelIds);
        }

        List<OverallTrendTop> overallTrendTop = chStatPlatformService.getOverallTrendTop(channelIdSet, startDate, endDate);
        if (overallTrendTop == null) {
            return new Result(GlobalApiError.OperationFailed, "暂无统计数据");
        }
        return new Result(overallTrendTop);
    }

    /**
     * 整体趋势 中间
     *
     * @param startDate
     * @param endDate
     * @param tagType
     * @return
     */
    @GET
    @Path("getOverallTrendMiddle")
    @Override
    public Result getOverallTrendMiddle(@QueryParam("startDate") Long startDate,
                                        @QueryParam("endDate") Long endDate,
                                        @QueryParam("tagType") Integer tagType) {

        if (startDate == null || endDate == null || tagType == null) {
            return new Result(GlobalApiError.BadRequest);
        }

        OverallTrendMiddle overallTrendMiddle = chStatPlatformService.getOverallTrendMiddle(this.getToken().getChannelId().toString(), startDate, endDate, tagType);
        if (overallTrendMiddle == null) {
            return new Result(GlobalApiError.OperationFailed, "暂无统计数据");
        }
        return new Result(overallTrendMiddle);
    }

    /**
     * 整体趋势 底部
     *
     * @param startDate
     * @param endDate
     * @param channelIdArray
     * @return
     */
    @GET
    @Path("getOverallTrendBottom")
    @Override
    public Result getOverallTrendBottom(@QueryParam("startDate") Long startDate,
                                        @QueryParam("endDate") Long endDate,
                                        @QueryParam("channelIdArray") String channelIdArray) {

        if (startDate == null || endDate == null) {
            return new Result(GlobalApiError.BadRequest);
        }

        Set<Long> channelIdSet = new HashSet<>();
        Long channelId = this.getToken().getChannelId();
        Long adminId = this.getToken().getAdminId();

        Set<Long> channelIds = new HashSet<>();
        // 母账号
        if (chAdminService.isParent(adminId)) {
            Long channelGroupId = chAdminService.getChannelGroupId(adminId);
            List<ChannelDTO> dtoList = chChannelService.getGroupChannelList(channelGroupId);
            channelIds.addAll(dtoList.stream().map(ChannelDTO::getChannelId).collect(Collectors.toSet()));
        } else {
            // 子账号
            String authChannelIds = chAdminService.getAuthChannelIds(adminId);
            if (StringUtils.isNotEmpty(authChannelIds)) {
                Set<Long> authChannelIdSet = Arrays.stream(authChannelIds.split(",")).map(e -> Convert.toLong(e)).collect(Collectors.toSet());
                channelIds.addAll(authChannelIdSet);
            }
        }

        if (!CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            String[] split = channelIdArray.split(",");
            for (String item : split) {
                if (StringUtils.isNotEmpty(item) && (!channelIds.contains(Convert.toLong(item)) && !Convert.toLong(item).equals(channelId))) {
                    return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
                }
                channelIdSet.add(Convert.toLong(item));
            }
        } else if (CollectionUtils.isEmpty(channelIds) && StringUtils.isNotEmpty(channelIdArray)) {
            if (!channelIdArray.equals(Convert.toStr(channelId))) {
                return new Result(GlobalApiError.ServerError, "非法操作，不能查看其它渠道数据");
            }
            channelIdSet.add(Convert.toLong(channelId));
        } else if (StringUtils.isEmpty(channelIdArray)) {
            channelIdSet.add(channelId);
            channelIdSet.addAll(channelIds);
        }

        Map<Long, List<OverallTrendBottom>> listMap = chStatPlatformService.getOverallTrendBottom(channelIdSet, startDate, endDate);
        if (listMap == null) {
            return new Result(GlobalApiError.OperationFailed, "暂无统计数据");
        }
        return new Result(listMap);
    }

    /**
     * 用户访问趋势
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @GET
    @Path("getUserAccessTrend")
    @Override
    public Result getUserAccessTrend(@QueryParam("startDate") Long startDate,
                                     @QueryParam("endDate") Long endDate) {
        Map<Long, List<UserAccessTrend>> listMap = chStatPlatformService.getUserAccessTrend(this.getToken().getChannelId().toString(), startDate, endDate);
        if (listMap == null) {
            return new Result(GlobalApiError.OperationFailed, "暂无统计数据");
        }
        return new Result(listMap);
    }

    /**
     * 用户付费趋势
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @GET
    @Path("getUserPaymentTrend")
    @Override
    public Result getUserPaymentTrend(@QueryParam("startDate") Long startDate,
                                      @QueryParam("endDate") Long endDate) {
        Map<Long, List<UserPaymentTrend>> trend = chStatPlatformService.getUserPaymentTrend(this.getToken().getChannelId().toString(), startDate, endDate);
        if (trend == null) {
            return new Result(GlobalApiError.OperationFailed, "暂无统计数据");
        }
        return new Result(trend);
    }

    /**
     * 渠道注册趋势
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @GET
    @Path("getChannelRegisterTrend")
    @Override
    public Result getChannelRegisterTrend(@QueryParam("startDate") Long startDate,
                                          @QueryParam("endDate") Long endDate) {
        Map<Long, List<ChannelRegisterTrend>> trend = chStatPlatformService.getChannelRegisterTrend(this.getToken().getChannelId().toString(), startDate, endDate);
        if (trend == null) {
            return new Result(GlobalApiError.OperationFailed, "暂无统计数据");
        }
        return new Result(trend);
    }
}
