package com.starbuds.server.api.channel.v1.impl;

import com.starbuds.server.api.channel.common.service.admin.ChUserService;
import com.starbuds.server.api.channel.v1.facade.UserFacade;
import com.starbuds.server.common.pojo.api.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

/**
 * 用户管理
 *
 * @author jiangxiaowei
 * @date 2020/4/22 14:27
 */
@Path("/v1/user")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class UserFacadeImpl extends BaseFacade implements UserFacade {

    @Resource
    ChUserService chUserService;

    /**
     * 用户列表
     *
     * @param userNo
     * @param userName
     * @param niceNo
     * @param userPhone
     * @param idCardNo
     * @param isAnchor
     * @param userStatus
     * @param pageNum
     * @param pageSize
     * @param startTime
     * @param endTime
     * @return
     */
    @GET
    @Path("getUserList")
    @Override
    public Result getUserList(@QueryParam("userNo") Long userNo,
                              @QueryParam("userName") String userName,
                              @QueryParam("niceNo") Long niceNo,
                              @QueryParam("userPhone") String userPhone,
                              @QueryParam("idCardNo") String idCardNo,
                              @QueryParam("isAnchor") Integer isAnchor,
                              @QueryParam("userStatus") Integer userStatus,
                              @QueryParam("pageNum") int pageNum,
                              @QueryParam("pageSize") int pageSize,
                              @QueryParam("sortType") int sortType,
                              @QueryParam("isAsc") int isAsc,
                              @QueryParam("startTime") Long startTime,
                              @QueryParam("endTime") Long endTime) {
        if (sortType == 0) {
            sortType = 1;
        }
        return new Result(chUserService.getUserList(userNo, userName, niceNo, userPhone, idCardNo, isAnchor, userStatus, this.getToken().getChannelId(), pageNum, pageSize, sortType, isAsc, startTime, endTime));
    }

    /**
     * 获取用户详情
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getUserDetail")
    @Override
    public Result getUserDetail(@QueryParam("userId") Long userId) {
        return new Result(chUserService.getUserDetail(userId));
    }
}
